# Script for converting P-A of the different tree species into a biomass estimate.
# Author: Robert W. Buchkowski
# Created: Jan, 19/2021

# Load in the libraries:
if (!require("pacman")) install.packages("pacman")
p_load(sf, fixest, tidyverse, sf, quantreg)

# Load the data  ----

# Compile the data set from forestry2 and the biomass estimates above
forestry_analysis = read_rds("Data/forest_inventory/forestry_compiled_biomass.rds") %>%
  filter(SITECODE %in% c("P","M", "G", "H")) %>%
  mutate(SITECODE = factor(SITECODE, levels = c("P","M", "G", "H"))) %>%
  filter(!is.na(temp)&!is.na(tempS)&!is.na(tempW)&!is.na(precipmm)&!is.na(precipSmm)&!is.na(precipWmm)&!is.na(elevation) & !is.na(bedrock) & !is.na(surfacerock) & !is.na(windspeed)) %>%
  filter(!is.na(HGT_CODE)) %>%
  mutate(HGT_CODE = as.numeric(HGT_CODE)) %>% # Get ride of all stands small enough to be browsed
  mutate(browsable = HGT_CODE == 1) %>%
  # Rescale all prediction variables
  mutate(temp = scale(temp, scale = F)[,1]) %>%
  mutate(tempS = scale(tempS, scale = F)[,1]) %>%
  mutate(tempW = scale(tempW, scale = F)[,1]) %>%
  mutate(precipmm = scale(precipmm, scale = F)[,1]) %>%
  mutate(precipSmm = scale(precipSmm, scale = F)[,1]) %>%
  mutate(precipWmm = scale(precipWmm, scale = F)[,1]) %>%
  mutate(elevation = scale(elevation, scale = F)[,1])%>%
  mutate(windspeed = scale(windspeed, scale = F)[,1])%>%
  # Split up the browsable and not browsable fir
  mutate(fir = ifelse(browsable, 0, bFbiomass)) %>%
  mutate(juvfir = ifelse(browsable, bFbiomass, 0)) %>%
  mutate(spruce = bSbiomass + wSbiomass)  %>%
  select(UID, SHAPE_Area, temp,tempS,tempW, precipmm, precipSmm, precipWmm, elevation, windspeed, NHN,bedrock,surfacerock,SITECODE,browsable, fir,juvfir, spruce) %>%
  filter(!is.na(fir) & !is.na(spruce)) %>%
  # Load in the moose management area ids and turn 000 into 006
  left_join(
    read_rds("Data/UIDinMMA.rds") %>%
      mutate(MMA = ifelse(MMA == "000", "006", MMA))
  )

# Log transform the data
forestry_analysis$firlog = log(forestry_analysis$fir + 1)
forestry_analysis$sprucelog = log(forestry_analysis$spruce + 1)
forestry_analysis$juvfirlog = log(forestry_analysis$juvfir + 1)

if(F) write_rds(forestry_analysis, "Data/forestry_analysis_tempsave.rds")

# Create prediction data frames ----
if(F){
  
  # Create a prediction data frame by NHN 
  forestry_analysis %>%
    group_by(NHN) %>%
    summarise(spruce_SD = sd(spruce), spruce_mean = weighted.mean(spruce,SHAPE_Area),
              fir_SD = sd(fir), fir_mean = weighted.mean(fir,SHAPE_Area),
              juvfir_SD = sd(juvfir), juvfir_mean = weighted.mean(juvfir,SHAPE_Area)) %>%
    pivot_longer(-NHN) %>%
    separate(name, into = c("ID", "name"), sep = "_") %>%
    pivot_wider() %>%
    write_rds("Data/FRI_trees_by_NHN.rds")
  
  # Create a prediction data frame by MMA 
  forestry_analysis %>%
    group_by(MMA) %>%
    summarise(spruce_SD = sd(spruce), spruce_mean = weighted.mean(spruce,SHAPE_Area),
              fir_SD = sd(fir), fir_mean = weighted.mean(fir,SHAPE_Area),
              juvfir_SD = sd(juvfir), juvfir_mean = weighted.mean(juvfir,SHAPE_Area)) %>%
    pivot_longer(-MMA) %>%
    separate(name, into = c("ID", "name"), sep = "_") %>%
    pivot_wider() %>%
    write_rds("Data/FRI_trees_by_MMA.rds")
  
}
# Plot some diagnostics -----
if(F){
  
  
  # Report summary statistics
  dim(forestry_analysis) # 969,946 commerical stants
  sum(forestry_analysis$SHAPE_Area)/10000 # 2,962,277 ha
  
  forest_temp <- read_rds("Data/forest_inventory/forestry_compiled_biomass.rds")
  
  dim(forest_temp) # 1,849,315 stands
  sum(forest_temp$SHAPE_Area)/10000 # 5,764,790 ha
  
  rm(forest_temp)
  
  MMA2 <- read_sf("C:/Users/rober/Documents/Moose_Model_Original_Data/MMA/MMA.shp") %>%
    mutate(MMA = ifelse(MMA == "000", "006", MMA)) %>%
    group_by(MMA) %>% 
    summarize()
  
  MMA2a = tibble(MMA = MMA2$MMA,
                 MMAarea = as.numeric(st_area(MMA2))) %>%
    full_join(
      forestry_analysis %>%
        group_by(MMA) %>%
        summarize(Area = sum(SHAPE_Area), N = n()), by = "MMA"
    ) %>%
    filter(!is.na(N)) %>%
    mutate(PropMMA = Area/MMAarea) %>%
    mutate(Area = Area/1000000,
           MMAarea = MMAarea/1000000) 
  
  MMA2a %>%
    arrange(PropMMA) %>% View()
  
  MMA2a %>%
    write_csv("Data/commerical_forest_per_MMA.csv")
  
  rm(MMA2a, MMA2)  
  
  # Take a look at the distribution of the biomass data
  png("Plots/biomass_dist.png", width = 12, height = 5, units = "in", res = 600)
  ggpubr::ggarrange(
    forestry_analysis %>% ggplot(aes(x = fir)) + geom_histogram() + theme_classic(),
    forestry_analysis %>% ggplot(aes(x = spruce)) + geom_histogram() + theme_classic(),
    forestry_analysis %>% ggplot(aes(x = juvfir)) + geom_histogram() + theme_classic(), nrow = 1, ncol = 3
  )
  dev.off()
  
  # Look at the distribution of abiotic variables by MMA and NHN
  png("Plots/abiotic_dist.png", width = 14, height = 5, units = "in", res = 600)
  forestry_analysis %>%
    select(MMA, temp, tempS, tempW, precipmm,precipSmm,precipWmm, elevation, windspeed) %>%
    pivot_longer(-MMA) %>%
    ggplot(aes(x = MMA, y = value)) + geom_boxplot() + theme_classic() + facet_wrap(.~name, scales = "free")
  dev.off()
  
  # Plot by NHN
  NHNplots = read_sf("Data/NHN/NHN.shp") %>%
    rename(NHN = NHNname) %>%
    left_join(
      read_rds("Data/FRI_trees_by_NHN.rds")
    ) 
  
  png("Plots/NHN_trees_mean.png", width = 8, height = 6, units = "in", res = 600)
  cowplot::plot_grid(
    NHNplots %>%
      filter(ID == "fir") %>%
      ggplot() + geom_sf(aes(fill = mean)) + scale_fill_viridis_c(name = "Balsam\nFir (t/ha)") + theme_classic(),
    NHNplots %>%
      filter(ID == "juvfir") %>%
      ggplot() + geom_sf(aes(fill = mean)) + scale_fill_viridis_c(name = "Juv. Balsam\nFir (t/ha)") + theme_classic(),
    NHNplots %>%
      filter(ID == "spruce") %>%
      ggplot() + geom_sf(aes(fill = mean)) + scale_fill_viridis_c(name = "Spruce\n(t/ha)") + theme_classic()
  )
  dev.off()
  
  # Plot by MMA:
  
  MMAplots = read_sf("C:/Users/rober/Documents/Moose_Model_Original_Data/MMA/MMA.shp") %>%
    mutate(MMA = ifelse(MMA == "000", "006", MMA)) %>%
    group_by(MMA) %>% 
    summarize() %>%
    left_join(
      read_rds("Data/FRI_trees_by_MMA.rds")
    ) 
  
  png("Plots/MMA_trees_mean.png", width = 8, height = 6, units = "in", res = 600)
  cowplot::plot_grid(
    MMAplots %>%
      filter(ID == "fir") %>%
      ggplot() + geom_sf(aes(fill = mean)) + scale_fill_viridis_c(name = "Balsam Fir (t/ha)") + theme_classic(),
    MMAplots %>%
      filter(ID == "juvfir") %>%
      ggplot() + geom_sf(aes(fill = mean)) + scale_fill_viridis_c(name = "Juv. Balsam\nFir (t/ha)") + theme_classic(),
    MMAplots %>%
      filter(ID == "spruce") %>%
      ggplot() + geom_sf(aes(fill = mean)) + scale_fill_viridis_c(name = "Spruce (t/ha)") + theme_classic()
  )
  dev.off()
  
}
# Run linear models relating biomass to abiotic variables ----

# Function to extract model information
MtoTIBBLE <- function(M, forplot = F){
  estimate = signif(M$coeftable[,1], 2)
  error = signif(M$coeftable[,2], 2)
  
  if(forplot){
    rtn = tibble(ID = rownames(M$coeftable),
                 EST = estimate,
                 error = error)
  }else{
    rtn = tibble(ID = rownames(M$coeftable),
                 EST = paste0(estimate, " (", error,")"))
  }
  return(rtn)
}

# The temperature and precipitation variables are highly correlated.
if(F) forestry_analysis %>% select(temp , tempS , tempW , precipmm , precipSmm , precipWmm , elevation , windspeed) %>% as.data.frame() %>% cor() %>% View()

# Summer and winter precip are highly correlated, so use annual. Summer and winter temperature are not, so use individual ones.

# Run the models:
# Watershed
variables = c("SITECODE","tempS","tempW","precipmm","elevation","windspeed","NHN","bedrock","surfacerock")

variables2 = expand.grid(variables, variables)
variables2 <- variables2[apply(variables2, 1, function(x) {length(unique(x)) == 2}),]


variables2 = c(paste0(combinat::combn(variables, 9), collapse = "+"),
               apply(combinat::combn(variables, 8),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 7),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 6),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 5),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 4),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 3),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 2),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 1),2, paste0, collapse = "+"))

SCGLMlist = vector(mode = "list", length = length(variables2))

for(i in 1:length(variables2)){
  SCGLMlist[[i]] <- MtoTIBBLE(
      feglm(formula(paste0("firlog~",variables2[i])),data = forestry_analysis,family = gaussian()), 
      forplot = T) %>%
    mutate(species = "Fir") %>%
    mutate(MID = i) %>%
    bind_rows(
      MtoTIBBLE(
        feglm(formula(paste0("sprucelog~",variables2[i])),data = forestry_analysis,family = gaussian()), 
        forplot = T) %>%
        mutate(species = "Spruce") %>%
        mutate(MID = i)
    ) %>%
    bind_rows(
      MtoTIBBLE(
        feglm(formula(paste0("juvfirlog~",variables2[i])),data = forestry_analysis,family = gaussian()), 
        forplot = T) %>%
        mutate(species = "JuvFir") %>%
        mutate(MID = i)
    )
  print(i)
}

p1 = do.call("rbind",SCGLMlist)

# MMA
variables = c("SITECODE","tempS","tempW","precipmm","elevation","windspeed","MMA","bedrock","surfacerock")

variables2 = expand.grid(variables, variables)
variables2 <- variables2[apply(variables2, 1, function(x) {length(unique(x)) == 2}),]


variables2 = c(paste0(combinat::combn(variables, 9), collapse = "+"),
               apply(combinat::combn(variables, 8),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 7),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 6),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 5),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 4),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 3),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 2),2, paste0, collapse = "+"),
               apply(combinat::combn(variables, 1),2, paste0, collapse = "+"))

SCGLMlist = vector(mode = "list", length = length(variables2))

for(i in 1:length(variables2)){
  SCGLMlist[[i]] <- MtoTIBBLE(
    feglm(formula(paste0("firlog~",variables2[i])),data = forestry_analysis,family = gaussian()), 
    forplot = T) %>%
    mutate(species = "Fir") %>%
    mutate(MID = i) %>%
    bind_rows(
      MtoTIBBLE(
        feglm(formula(paste0("sprucelog~",variables2[i])),data = forestry_analysis,family = gaussian()), 
        forplot = T) %>%
        mutate(species = "Spruce") %>%
        mutate(MID = i)
    ) %>%
    bind_rows(
      MtoTIBBLE(
        feglm(formula(paste0("juvfirlog~",variables2[i])),data = forestry_analysis,family = gaussian()), 
        forplot = T) %>%
        mutate(species = "JuvFir") %>%
        mutate(MID = i)
    )
  print(i)
}

p2 = do.call("rbind",SCGLMlist)

# Combine the two types of spatial subsetting:

pall = p1 %>%
  mutate(SpatialGroup = "Watershed") %>%
  bind_rows(
    p2 %>%
      mutate(SpatialGroup = "MMA")
  )
rm(p1, p2)

pall %>%
  group_by(ID, species, SpatialGroup) %>%
  summarize(CV = abs(sd(EST)/mean(EST))) %>%
  ungroup() %>%
  mutate(GR = paste0(species, SpatialGroup)) %>%
  select(ID, GR, CV) %>%
  pivot_wider(names_from = GR, values_from = CV) %>%
  View()

pall %>%
  filter(ID == "tempW" & species == "Fir" & EST < 0)

# MAKE PLOTS:

png("Plots/glm_coefficients_target_fir.png", width = 12, height = 8, units = "in", res = 600)
 pall %>% 
   filter(!grepl("MMA", ID) & !grepl("NHN", ID) & species == "Fir") %>%
   ggplot(aes(x = EST, fill = SpatialGroup)) + geom_histogram() + facet_wrap(~ID, scales = "free") + theme_classic() 
dev.off()

png("Plots/glm_coefficients_target_spruce.png", width = 12, height = 8, units = "in", res = 600)
pall %>% 
  filter(!grepl("MMA", ID) & !grepl("NHN", ID) & species == "Spruce") %>%
  ggplot(aes(x = EST, fill = SpatialGroup)) + geom_histogram() + facet_wrap(~ID, scales = "free") + theme_classic() 
dev.off()

png("Plots/glm_coefficients_target_juvfir.png", width = 12, height = 8, units = "in", res = 600)
pall %>% 
  filter(!grepl("MMA", ID) & !grepl("NHN", ID) & species == "JuvFir") %>%
  ggplot(aes(x = EST, fill = SpatialGroup)) + geom_histogram() + facet_wrap(~ID, scales = "free") + theme_classic() 
dev.off()

# Final model selection:
NBF <- feglm(firlog~SITECODE + precipmm + tempS + MMA,data = forestry_analysis,family = gaussian())

NBF0 <- feglm(firlog~SITECODE + precipmm + tempS + MMA,data = forestry_analysis %>% filter(juvfir == 0 & spruce == 0),family = gaussian())


NBF2 <- feglm(firlog~SITECODE + precipmm + tempS + NHN,data = forestry_analysis,family = gaussian())

AS <- feglm(sprucelog~SITECODE + precipmm + tempS + MMA,data = forestry_analysis,family = gaussian())

BF <- feglm(juvfirlog~SITECODE + precipmm + tempS + MMA,data = forestry_analysis,family = gaussian())

coefficients(NBF)
coefficients(AS)
coefficients(BF)

paste0(paste0(coefficients(BF)[7:52], "*(site['MMA']==",c(2:39, 46, 40:45, 47), ")"), collapse = "+")

