# Extract the watershed temperature, precipitation, and site quality data ----

# Author: Robert W. Buchkowski
# Created: Feb 23/2021
# Modified: Feb 23/2021

# Load in the libraries:
if(!require("pacman")) install.packages("pacman")
p_load(sf, raster, ggspatial, tidyverse)

# Read in the elevation data
elevation = raster("Data/Elevation/elevation.grd")

# Temperature (need to divide by 10)
tempread <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_01.tif")

tempread = calc(tempread, function(x){x/10})

# Precipitation
precipread <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_12.tif")

# Average wind speed
windspeed <- raster("Data/windspeed/windspeed.tif")

# Temperature (need to divide by 10)
ASTread <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_10.tif")

ASTread = calc(ASTread, function(x){x/10})

AWTread <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_11.tif")

AWTread = calc(AWTread, function(x){x/10})

# Precipitation
TWPread <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_18.tif")

TCPread <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_19.tif")

# Combine the temperature and precipitation rasters
tempprecip <- brick(tempread,ASTread,AWTread, precipread,TWPread,TCPread)
rm(tempread, precipread)

# Crop the raster to the right extent
temprecip <- crop(tempprecip,elevation)
rm(tempprecip, elevation)
gc()

# Load in the spatial data sets
MMA <- read_sf("C:/Users/rober/Documents/Moose_Model_Original_Data/MMA/MMA.shp") %>%
  mutate(MMA = ifelse(MMA == "000", "006", MMA)) %>%
  group_by(MMA) %>% 
  summarize()

# Look at the positions of the small MMAs
MMA %>%
  filter(MMA %in% c("002","002A","002B","005", "005A", "GMNP")) %>%
  ggplot() + geom_sf(aes(fill = MMA))

# Calculate the means for temperature and precipitation to correct them appropriately (See statistical_models_biomass.R)

read_rds("Data/forest_inventory/forestry_compiled_biomass.rds") %>%
  filter(SITECODE %in% c("P","M", "G", "H")) %>%
  mutate(SITECODE = factor(SITECODE, levels = c("P","M", "G", "H")))%>%
  filter(!is.na(temp)&!is.na(precipmm)&!is.na(elevation) & !is.na(bedrock) & !is.na(surfacerock)) %>%
  filter(!is.na(HGT_CODE)) %>%
  # Calculate the means
  summarize(temp_mean = mean(temp),
            tempS_mean = mean(tempS),
            tempW_mean = mean(tempW),
            precip_mean = mean(precipmm),
            precipS_mean = mean(precipSmm),
            precipW_mean = mean(precipWmm),
            ws_mean = mean(windspeed), 
            temp_min = min(temp), 
            temp_max = max(temp), 
            precip_min = min(precipmm), 
            precip_max = max(precipmm), 
            ws_min = min(windspeed), 
            ws_max = max(windspeed)) %>% View()

# Mean temperature is 3.84 with range 0.6 to 6
# Mean summer temperature is 14.6564
# Mean winter temperature is -6.808112
# Mean precipitation is 1219.023 with range 920 to 1684
# Mean summer precipitation is 1217.831
# Mean winter precipitation is 1217.825
# Mean wind speed is 22.98008 with range 19.72 to 30.95

# Extract the temperature and precipitation data for each watershed to prepare a distribution

MMA_extract <- raster::extract(temprecip, # Use the NL only temperature and precipitation data
                               MMA,
                               method = "simple", 
                               small = T, # Allow the code to find a possible value outside the polygon if it doesn't cover a raster cell
                               fun = mean,
                               sp = T,
                               na.rm = T
)


# Extract wind speed
MMA_extract <- raster::extract(windspeed, # Use the NL only temperature and precipitation data
                                    MMA_extract,
                                    method = "simple", 
                                    small = T, # Allow the code to find a possible value outside the polygon if it doesn't cover a raster cell
                                    fun = mean,
                                    sp = T,
                               na.rm = T
)

# Get the MODE for Site codes:
FA = read_rds("Data/forest_inventory/forestry_compiled_biomass.rds") %>%
  filter(SITECODE %in% c("P","M", "G", "H")) %>%
  mutate(SITECODE = factor(SITECODE, levels = c("P","M", "G", "H")))%>%
  filter(!is.na(temp)&!is.na(precipmm)&!is.na(elevation) & !is.na(bedrock) & !is.na(surfacerock)) %>%
  filter(!is.na(HGT_CODE)) %>%
  left_join(
    read_rds("Data/UIDinMMA.rds"), by = "UID"
  ) %>%
  select(UID, MMA, SITECODE)

head(FA)

FA %>%
  mutate(MMA = ifelse(MMA == "000", "006", MMA)) %>%
  group_by(MMA, SITECODE) %>%
  summarize(N = n()) %>%
  pivot_wider(names_from = SITECODE, values_from = N) %>%
  mutate(H = replace_na(H,0)) %>%
  full_join(
    FA %>%
      mutate(MMA = ifelse(MMA == "000", "006", MMA)) %>%
      group_by(MMA, SITECODE) %>%
      summarize(N = n()) %>%
      slice_max(N) %>%
      select(-N) %>%
      rename(Qualmode = SITECODE)
  ) %>%
  full_join(
    MMA_extract %>%
      as_tibble() %>%
      rename(bio01 = layer.1,
             bio10 = layer.2,
             bio11 = layer.3,
             bio12 = bio60_12,
             bio18 = bio60_18,
             bio19 = bio60_19), by = "MMA"
  ) %>%
  mutate(temperature = bio01 - 3.84,
         summertemperature = bio10 - 14.6564,
         wintertemperature = bio11 + 6.808112,
         precipitation = bio12 - 1219.023,
         summerprecipitation = bio18 - 1217.831,
         winterprecipitation = bio19 - 1217.825,
         windspeed = windspeed - 22.98008) %>%
  write_rds("Data/MMA_abiotic_variables.rds")
