# Forest Inventory to MMA
# Author: Robert W. Buchkowski
# Date: Jan 25/2021

# The goal of this analysis is to match the forest inventory plots to MMA across the island of Newfoundland. The output is a file with UID and MMA

# Load in the libraries:
if(!require("pacman")) install.packages("pacman")
p_load(tidyverse, sf)

# Load in the MMA shape files

MMA <- read_sf("C:/Users/rober/Documents/Moose_Model_Original_Data/MMA/MMA.shp")
MMA %>% ggplot() + geom_sf() + theme_classic() + scale_fill_viridis_c()

# Load in the forestry spatial database
forestryshape <- read_sf("Data/forest_inventory/inventory_spatial/forestry1.shp") %>%
  bind_rows(
    read_sf("Data/forest_inventory/inventory_spatial/forestry2.shp")
  ) %>%
  bind_rows(
    read_sf("Data/forest_inventory/inventory_spatial/forestry3.shp")
  )

# Find the centroids for each forestry polygon for mapping to MMA:
forestryshape <- st_centroid(forestryshape)

MMA2 = MMA %>% select(MMA)

MMA2 = st_transform(MMA2, st_crs(forestryshape))

# Find the location of each forestry centroid in each MMA
inventoryMMA <- st_join(
  forestryshape,
  MMA2
)

tibble(inventoryMMA) %>%
  select(MMA, UID) %>%
  write_rds("Data/UIDinMMA.rds")

# Check for repeats:
read_rds("Data/UIDinMMA.rds") %>%
  group_by(UID) %>%
  summarize(N = n()) %>%
  filter(N > 1) # One repeat

forestryshape %>% filter(UID == 301799) %>%
  ggplot() + geom_sf(data = MMA2) + geom_sf(size = 3, col = "red") 

read_rds("Data/UIDinMMA.rds") %>%
  filter(UID == 301799)

# Load in the polygon
a = st_read("C:/Users/rober/Documents/GitHubRepos/MooseEcon/Data/forest_inventory/inventory_spatial/forestry1.shp", query = 'SELECT * FROM "forestry1" WHERE UID = 301799')

MMA2 %>%
  filter(MMA %in% c("013", "017")) %>%
  ggplot() + geom_sf() + geom_sf(data = a, size = 1.2, col = "red")

b = st_join(a,
            MMA2 %>%
              filter(MMA %in% c("013", "017"))
)

st_area(b$`_ogr_geometry_`)

# It is literally in the overlap between 013 and 017. Just randomly assign it to one sample:
sample(c("013", "017"), 1) # Assigned to 013

# Remove that row from the data set
read_rds("Data/UIDinMMA.rds")  %>%
  filter(!(UID == 301799 & MMA == "017")) %>%
  write_rds("Data/UIDinMMA.rds")

# Identify the points that have no MMA
nomatches = read_rds("Data/UIDinMMA.rds") %>%
  filter(is.na(MMA)) %>%
  select(UID)

nomatches = 
  forestryshape %>% filter(UID %in% nomatches$UID)

# Remove the 99 empty plots
nomatches = nomatches[!is.na(st_dimension(nomatches)),]

# Look at the distribution of unmatched plots. Most of these are on the coast or offshore islands that are not part of the MMA in the same way. I added them to the nearest MMA
MMA2 %>%
  ggplot() + geom_sf() + geom_sf(data = nomatches, size = 1.2, col = "red")

# I can easily match these up with nearest distance
matches <- tibble(UID = nomatches$UID, MMA = NA)

for(i in 1:dim(nomatches)[1]){
  dist = st_distance(MMA2, nomatches[i,])
  matches$MMA[i] = MMA2$MMA[which.min(dist)]
  if(i %% 100 == 0) print(paste("Done", i, "of 19700"))
}

matches$flagMMA = 2

ff <- read_rds("Data/UIDinMMA.rds") %>%
  mutate(flagMMA = 1) %>%
  filter(!(UID %in% matches$UID)) %>%
  bind_rows(
    matches
  ) 

# Now only the 99 are left
table(is.na(ff$MMA))

ff %>%
  write_rds("Data/UIDinMMA.rds")

# Look at the National Park matches:

ff %>%
  filter(MMA %in% c("GMNP", "TNNP"))

forestryshape %>%
  right_join(
    ff %>%
      filter(MMA %in% c("GMNP"))
    ) %>% 
  mutate(ID = "Stand") %>%
  bind_rows(
    MMA %>%
      filter(MMA %in% c("GMNP", "005")) %>%
      mutate(ID = "MMA")
  ) %>%
  ggplot(aes(color = ID)) + geom_sf()

# Move GMNP forest stands to 005 where they belong.
ff = ff %>%
  mutate(MMA = ifelse(MMA == "GMNP", "005", MMA))

# Check TNNP
forestryshape %>%
  right_join(
    ff %>%
      filter(MMA %in% c("TNNP"))
  ) %>% 
  mutate(ID = "Stand") %>%
  bind_rows(
    MMA %>%
      filter(MMA %in% c("TNNP", "028")) %>%
      mutate(ID = "TNNP")
  ) %>%
  ggplot(aes(color = ID)) + geom_sf()

# Move the TNNP stands to 028
ff = ff %>%
  mutate(MMA = ifelse(MMA == "TNNP", "028", MMA))

# Confirm the parks are gone from the data set where placed incorrectly
ff %>% filter(MMA %in% c("TNNP", "GMNP"))

# Re-write the matching data frame
ff %>%
  write_rds("Data/UIDinMMA.rds")

# Clean up the environment
rm(list = ls()); gc()
