# Calculating harvest rates:
# Author: Robert W. Buchkowski
# Created: Feb 16/2021

# The goal is to calculate reasonable values for the harvesting and hunting parameters

# Load in the necessary libraries:
if(!require("pacman")) install.packages("pacman")
p_load(tidyverse, lubridate, sf, googledrive)

# Harvesting ----

# The timber resource guide for 2015-2020 provide the following table of annual allowable cuts by district:

# We know an average of 8400 ha were harvested annually

# Read in the FRI data
FRI = read_rds("Data/forest_inventory/forestry_compiled_biomass.rds")  %>%
  left_join(
    read_rds("Data/UIDinFMD.rds")
  ) %>%
  group_by(FMD) %>%
  summarize(bFbiomass = mean(bFbiomass, na.rm = T),
            bSbiomass  = mean(bSbiomass, na.rm = T),
            wSbiomass  = mean(wSbiomass, na.rm = T),
            area = sum(SHAPE_Area, na.rm = T)/10000)

# Read in the FMD data and join it with the AAC and FRI data
prop = tibble(FMD = c(1:18, 5,6,9,14,15,16),
       Crown = c(rep(1, 18), rep(0, 6)),
       AAC = c(122485,101244,13391,55760,97407,66439,25818,98480,94218,77945,174675,186550,19229,93644,54571,88232,122800,126659,
               53326,69342,169248,122262,283431,139400)
       ) %>%
  group_by(FMD) %>%
  summarize(AAC = sum(AAC)) %>%
  full_join(
    # Read in the biomass data by district and join it. The units are tonnes per hectare
    
    tibble(read_sf("Data/FMDmap/FMDmap.shp")) %>%
      select(FMD, BalsmFr, BlckSpr) %>%
      mutate(FMD = as.numeric(FMD)) %>%
      mutate(hectares = as.vector(st_area(read_sf("Data/FMDmap/FMDmap.shp")))/10000)
    
  ) %>%
  full_join(
    FRI %>% mutate(FMD = as.numeric(FMD)) %>%
      mutate(Sbiomass = bSbiomass + wSbiomass) %>%
      mutate(bFbiomass = bFbiomass*area,
             Sbiomass = Sbiomass*area) %>%
      select(-bSbiomass, -wSbiomass, -area)
  ) %>%
  filter(!is.na(BlckSpr)) %>%
  # Calculate the number of tonnes on average in the FMD
  mutate(B = BalsmFr*hectares,
         S = BlckSpr*hectares) %>%
  
  # NEED TO FIGURE OUT THE AMOUNT OF FORESTED HECTARES BY FMD, could used the FRI database to determine this.
  
  # Convert to m3 to match the AAC units:
  # Groot, A., and J. E. Luther. 2015. Hierarchical analysis of black spruce and balsam fir wood density in Newfoundland. Canadian Journal of Forest Research 45:805–816.
  # 436.5/1000 # kg/m3 BF converted to tonnes
  # 570.9/1000 # kg/m3 BS converted to tonnes
  mutate(B2 = bFbiomass*1000/436.5, # Using the FRI version
         S2 = Sbiomass*1000/570.9) %>% # Using the FRI version
  mutate(propAAC_B = AAC/(2*B2),
         propAAC_S = AAC/(2*S2))


# Map AAC by FMD to MMA using the individual stands and distributing cutting equally across the FMD
entiredata = read_rds("Data/forest_inventory/forestry_compiled_biomass.rds")  %>%
  left_join(
    read_rds("Data/UIDinFMD.rds")
  ) %>% 
  left_join(
    read_rds("Data/UIDinMMA.rds") %>%
      mutate(MMA = ifelse(MMA == "000", "006", MMA))
  ) %>%
  left_join(
    # Get the proportion of biomass cut by FMD
    prop %>% 
      select(FMD, propAAC_B, propAAC_S) %>%
      mutate(FMD = str_pad(as.character(FMD),2, "left", "0")) %>%
      bind_rows(
        tibble(FMD = c("50", "03"), # 50 is GMNP and TNNP and 03 has little productive forest according to the forestry department
               propAAC_B = c(0,0),
               propAAC_S = c(0,0))
      )
    
  )

# Why are there forestry plots in the park?
entiredata %>% filter(MMA == "GMNP") %>% View()
entiredata %>% filter(MMA == "TNNP") %>% View()

# Load in the MMA shape files

MMA <- read_sf("C:/Users/rober/Documents/Moose_Model_Original_Data/MMA/MMA.shp")

# Load in the forestry spatial database
forestryshape <- read_sf("Data/forest_inventory/inventory_spatial/forestry1.shp") %>%
  bind_rows(
    read_sf("Data/forest_inventory/inventory_spatial/forestry2.shp")
  ) %>%
  bind_rows(
    read_sf("Data/forest_inventory/inventory_spatial/forestry3.shp")
  )


UIDsel = entiredata %>% filter(MMA %in% c("GMNP", "TNNP"))
UIDsel = UIDsel$UID

UIDselGMNP = entiredata %>% filter(MMA %in% c("GMNP"))
UIDselGMNP = UIDselGMNP$UID

UIDselTNNP = entiredata %>% filter(MMA %in% c("TNNP"))
UIDselTNNP = UIDselTNNP$UID

forestryshape = 
  forestryshape %>%
  filter(UID %in% UIDsel)

MMA %>% filter(MMA == "GMNP") %>% ggplot() + geom_sf() + theme_classic() + geom_sf(data = forestryshape %>% filter(UID %in% UIDselGMNP), color = "red", size = 2)


MMA %>% filter(MMA == "TNNP") %>% ggplot() + geom_sf() + theme_classic() + geom_sf(data = forestryshape %>% filter(UID %in% UIDselTNNP), color = "red", size = 2)


# Group by MMA
entiredata2 = entiredata %>%
  group_by(MMA) %>%
  
  # Weight by shape area 
  summarize(
    propAAC_B = weighted.mean(propAAC_B, SHAPE_Area, na.rm = T),
    propAAC_S = weighted.mean(propAAC_S, SHAPE_Area, na.rm = T )
    ) 

# Make TNNP and GMNP zero: some edge stands are getting included by mistake because the FMD and MMA park boundaries don't exactly overlap
entiredata2[entiredata2$MMA %in% c("GMNP", "TNNP"),2:3] = 0

entiredata2 %>%
  write_rds("Data/harvest_by_MMA.rds")


prop %>%
  ggplot(aes(x = bFbiomass/1e4, y = B/1e4)) + geom_point() + geom_abline(intercept = 0, slope = 1)

prop %>%
  ggplot(aes(x = Sbiomass/1e4, y = S/1e4)) + geom_point() + geom_abline(intercept = 0, slope = 1)

prop  %>%
  pivot_longer(contains("prop")) %>%
  group_by(name) %>%
  summarise(sd(value), mean(value), min(value), max(value))

# See if the 8400 ha total comes out to about the right value

prop %>%
  summarize(B = sum(BalsmFr), S = sum(BlckSpr), ha = sum(hectares))

# Calculate the planting rates of spruce -----

FRI = read_rds("Data/forest_inventory/forestry_compiled_biomass.rds")

if(F){
  unique(FRI$TYPE_SILV)
  unique(FRI$SPECIES_SILV)
  
  
  FRI %>%
    filter(TYPE_SILV %in% c("P", "GP")) %>%
    mutate(Year = year(YEAR_SILV)) %>%
    group_by(SPECIES_SILV,Year) %>%
    summarize(area = sum(SHAPE_Area)/10000) %>% View()
  
  FRI %>%
    filter(TYPE_SILV %in% c("P", "GP")) %>%
    mutate(Year = year(YEAR_SILV)) %>%
    group_by(SPECIES_SILV,Year) %>%
    summarize(area = sum(SHAPE_Area)/10000) %>%
    group_by(Year) %>%
    summarize(area = sum(area)) %>% View()
  
  FRI %>%
    filter(TYPE_SILV %in% c("P", "GP")) %>%
    mutate(Year = year(YEAR_SILV)) %>%
    group_by(Year) %>%
    summarize(area = sum(SHAPE_Area)/10000) %>%
    ungroup() %>%
    summarize(area = mean(area)) # This analysis suggests an average of 3377 hectares planted over the data set
  
  FRI %>%
    filter(TYPE_SILV %in% c("P", "GP")) %>%
    mutate(Year = year(YEAR_SILV)) %>%
    group_by(Year) %>%
    summarize(area = sum(SHAPE_Area)/10000) %>%
    filter(Year > 2010 & Year < 2016) %>%
    ungroup() %>%
    summarize(area = mean(area)) # This analysis suggests an average of 4403 hectares planted from 2011 to 2015
  
  # an average of 4,530 hectares were planted per year between 2011 and 2015: Timber Resource Manual
  
  FRI %>%
    filter(TYPE_SILV %in% c("P", "GP")) %>%
    mutate(Year = year(YEAR_SILV)) %>%
    group_by(SPECIES_SILV,Year) %>%
    summarize(area = sum(SHAPE_Area)/1000) %>%
    group_by(SPECIES_SILV) %>%
    summarize(area = mean(area)) %>%
    mutate(freq = area/sum(area))
  # Spruce represents ~71% of the planting in NFLD.
  
}

# https://cfs.nrcan.gc.ca/pubwarehouse/pdfs/32450.pdf
# 1.5 to 1.8 meter spacing for black spruce planting

# Spruce seeding weight: https://cdnsciencepub.com/doi/pdf/10.1139/x97-207
# estimate 0.75g aboveground per seedling.

# Maybe from here: https://tspace.library.utoronto.ca/bitstream/1807/11705/1/MQ29459.pdf
# estimate 0.5g aboveground per seedling.

# Area of planting spruce in meters^2 BY MMA
FRITOT = FRI %>% 
  left_join(
    read_rds("Data/UIDinMMA.rds") %>%
      mutate(MMA = ifelse(MMA == "000", "006", MMA))
  )
  
FRITOT2 = 
  FRITOT  %>%
  mutate(Year = year(YEAR_SILV)) %>%
  # Get rid of all the forestry stands that are not considered in our analysis--those stands that are not even scored as productive. These are the same stands used to fit the glms for spruce and fir biomass
  filter(SITECODE %in% c("P","M", "G", "H")) %>%
  filter(!is.na(temp)&!is.na(precipmm)&!is.na(elevation) & !is.na(bedrock) & !is.na(surfacerock) & !is.na(windspeed)) %>%
  filter(!is.na(HGT_CODE)) %>%
  # Planting for spruce only
  mutate(Planting = ifelse(
    TYPE_SILV %in% c("P", "GP") & 
      SPECIES_SILV %in% c("wS", "bS"), "Yes","No")) %>%
  group_by(Planting, Year, MMA) %>%
  summarize(SHAPE_Area = sum(SHAPE_Area)) %>%
  pivot_wider(names_from = Year, values_from = SHAPE_Area) %>%
  rename(NOPLANT = `NA`) %>%
  pivot_longer(!contains("Planting") & !contains("MMA") & !contains("NOPLANT")) %>%
  mutate(value = replace_na(value, 0)) %>%
  mutate(NOPLANT = replace_na(NOPLANT, 0)) %>%
  mutate(value = NOPLANT + value) %>% # Add together UIDs with no planting ever into the area where planting didn't happen in each year
  select(-NOPLANT) %>%
  pivot_wider(names_from = Planting) %>%
  mutate(PropPlanted = Yes/(Yes + No)) %>% # Calculated the proportion of the forested area in each MMA that was planted with spruce per year
  group_by(MMA) %>%
  summarize(PropPlanted = mean(PropPlanted)) %>% # Calculated per year
  # Add in the min, med, and max rate of planting in tonnes per hectare
  mutate(PropPlanted = replace_na(PropPlanted, 0)) %>%
  mutate(min = 1/(2.5*2.5)* 10000 * 0.625 /1e6,
         med = 1/(2.1*2.1)* 10000 * 3.76/2 /1e6,
         max = 1/(2*2)* 10000 * 6.76/2 /1e6) %>%
  pivot_longer(min:max) %>%
  mutate(value = PropPlanted*value) %>%
  select(-PropPlanted) %>%
  pivot_wider()

# Add the planting rates for spruce into the harvest data frame:

read_rds("Data/harvest_by_MMA.rds") %>%
  select(-ps_min, -ps_med, -ps_max) %>%
  full_join(
    FRITOT2 %>% rename(ps_min = min, ps_med = med, ps_max = max)
  ) %>%
  write_rds("Data/harvest_by_MMA.rds")
