# Forestry Inventory at the FMD Level
# Author: Robert W. Buchkowski
# Created: Jan, 14/2021
# Modified: Feb 22/2021

# GOAL: Scrape inventory data from the most recent publicly available data. Extract temperature, precipitation, and elevation data by FMD. Produce a final output file with all these data in it. 

# FMD Data Source:
# https://www.gov.nl.ca/ffa/programs-and-funding/forestry-programs-and-funding/managing/inv-plan/

# Load the necessary packages:
library(sf)
library(ggspatial)
library(raster)
library(tidyverse)
mkplot = F

# Code to extract the data from PDF
if(F){
  # Extract the data from PDF and convert to CSV (this only needs to happen once. I ran this code on a MAC to avoid having to deal with rJava on windows).
  # library(tabulizer)
  
  ppath = "/Users/robertbuchkowski/Dropbox (Personal)/Transfer Folder/FMD_inventory"
  
  LF <- list.files(ppath)
  
  for(i in 1:length(LF)){
    assign(paste0("df", i), extract_areas(paste0(ppath, "/", LF[i])))
  }
}

# Compile the forest inventory data ----

# This section loads the separate CSV files extracted from PDF and combines them to summarize the forestry inventory data

fl <- list.files("C:/Users/rober/Documents/Moose_Model_Original_Data/FMD/CSV")

aa = vector("list",length = length(fl))

for(i in 1:length(fl)) aa[[i]] = read.csv(paste0("C:/Users/rober/Documents/Moose_Model_Original_Data/FMD/CSV/",fl[i]))

bb = do.call("rbind",aa)

bb2 = bb %>%
  left_join(
    data.frame(SPECIES = sort(unique(bb$SPECIES)),
               SPECIES_TYPE = c("SW",  "SW" , "SW" , "SW" , "SW", "SW", "HW", "HW", "HW", "HW", "HW", "HW"),
               SPECIES_NAME = c("White Pine",
                                "Other Pine",
                                "Black Spruce",
                                "White Spruce",
                                "Balsam Fir",
                                "Tamarack",
                                "Trembling Aspen",
                                "Other Poplar",
                                "White Birch",
                                "Yellow Birch",
                                "Red Maple",
                                "Other Hardwood"))
    )


OK = apply(bb2[,6:13],1, sum) == bb2$TOTAL

OK2 = apply(bb2[,6:13],1, sum) - bb2$TOTAL > 1e-6

bb2[OK2,]

bb2[,"TOTAL2"] = apply(bb2[,6:13],1, sum)

if(mkplot){
  bb2 %>%
    group_by(DISTRICT, SPECIES_NAME) %>%
    summarize(TOTAL = mean(TOTAL)) %>%
    filter(SPECIES_NAME == "Balsam Fir"|SPECIES_NAME == "Black Spruce"|SPECIES_NAME == "White Spruce") %>%
    ggplot(aes(x= TOTAL, color = SPECIES_NAME)) + geom_density() +theme_classic()
  
  bb2 %>%
    group_by(DISTRICT, SPECIES_NAME) %>%
    summarize(TOTAL = mean(TOTAL)) %>%
    filter(SPECIES_NAME == "Black Spruce") %>%
    ggplot(aes(x= TOTAL)) + geom_density() +theme_classic()
  
  # Species biomass (Tonnes/ha) across insular NFLD
  bb2 %>%
    group_by(SPECIES_NAME) %>%
    summarize(SD = sd(TOTAL),TOTAL = mean(TOTAL))
  
  bb2 %>%
    select(DISTRICT, Plot) %>%
    distinct() %>%
    group_by(DISTRICT) %>%
    summarize(n())
}

rm(fl, i, OK, OK2, aa, bb)

# Build the FMD dataset ----

# Load in insular FMD (forest management districts) NFLD districts
FMDmap <- read_sf("C:/Users/rober/Documents/Moose_Model_Original_Data/FMD/ForestryAdmin_N/FMD_Bndry_NL.shp") %>% 
  separate(FMD, into= c(NA, "FMD"), sep = " ") %>%
  separate(PLAN_ZONE, into= c(NA, "PLAN_ZONE"), sep = " ") %>%
  mutate(ID = paste0(FMD,":", PLAN_ZONE))

# Format and combined with forest inventory data
bb2 = bb2 %>% rename(FMD = DISTRICT) %>% mutate(FMD = str_pad(as.character(FMD), 2,"left", "0")) %>%
  group_by(FMD, SPECIES_NAME) %>%
  summarize(Total = mean(TOTAL)) %>%
  spread(key = SPECIES_NAME, value = Total)

# Combine the two data sets
FMDmap = FMDmap %>% left_join(bb2)

# Fix the geometry of the FMD
FMDmap = st_make_valid(FMDmap)

# Output the shape4 file
if(!dir.exists("Data/FMDmap")) dir.create("Data/FMDmap")
write_sf(FMDmap, "Data/FMDmap/FMDmap.shp")

# Only need to read this in if I am starting here.
if(F) FMDmap = read_sf("Data/FMDmap/FMDmap.shp")

png("Plots/FMD_fir.png", width = 8, height = 8, units = "in", res = 600)
FMDmap %>%
  ggplot() + geom_sf(aes(fill = BalsmFr)) + scale_fill_viridis_c(name = "Balsam Fir\n(tonnes/ha)") + theme_classic()
dev.off()

# STOPPED WORK HERE
# NEXT STEP IS TO EXTRACT THE CLIMATE GRIDS AND ELEVATION
                    















# [CAN BE SKIPPED IF FMD_abiotic FILE EXISTS] Downloading and Extracting temperature and precipitation data ----

# Start with the 30 year average data to begin the analysis time invariant

if(F){
  LF = dir("C:/Users/rober/Documents/GitHubRepos/Canada_climate_data_MooseEcon/AVERAGE30YEAR")
  LF = LF[grepl("bio",LF)]
  
  # Start with bioclim data
  for(i in 1:length(LF)){
    unzip(paste0("C:/Users/rober/Documents/GitHubRepos/Canada_climate_data_MooseEcon/AVERAGE30YEAR/", LF[i]), exdir = paste0(getwd(), "/AVERAGE30YEAR"))
  }
}

LF = list.files(paste0(getwd(), "/AVERAGE30YEAR"), recursive = T)

LF = LF[grepl(".tif",LF)] # Take only the .tif files

# Annual mean temperature
tempread <- raster(paste0(getwd(), "/AVERAGE30YEAR/", LF[1]))
plot(tempread)

# Annual precipitation
precipread <- raster(paste0(getwd(), "/AVERAGE30YEAR/", LF[12]))
plot(precipread)

FMDt2 = st_transform(shape4,crs(tempread)) # Convert FMD to T&P coordinate reference system

# Crop the temperature and precipitation rasters to insular NL
tempread_2 = crop(tempread, extent(FMDt2))
precipread_2 = crop(precipread, extent(FMDt2))

tempread_2C = df_spatial(tempread_2)
tempread_2C$band1 = tempread_2C$band1/10 # Convert to Celsius. Data is stored as degC x 10 to save memory in the raster

p_temperature = ggplot(FMDt2) + theme_classic() + geom_raster(aes(x = x, y= y, fill = band1), data = tempread_2C) + geom_sf(fill = NA, color = "white") + 
  scale_fill_gradient(low = "blue", high = "red", name = "Temperature (\u00B0C)") + theme(legend.position ="top")

p_precipitation = ggplot(FMDt2) + theme_classic() + geom_raster(aes(x = x, y= y, fill = band1), data = df_spatial(precipread_2)) + geom_sf(fill = NA, color = "white")  + 
  scale_fill_gradient(low = "brown", high = "green", name = "Precipitation (mm)") + theme(legend.position ="top")

if(mkplot){
  png("Plots/BIOCLIM_FMD.png", width = 10, height = 10, units = "in", res = 600)
  cowplot::plot_grid(p_temperature,p_precipitation)
  dev.off()
}

# Extract the climate data to FMD to determine the relationship between tree density and the climate variables
t1 = Sys.time()
ext_temp <- raster::extract(tempread_2, # Use the NL only temperature data
               FMDt2, # Use the FMD that is reprojected into the right coordinate system for the raster
               method = "simple") # takes a short time (~20 seconds)
t2 = Sys.time()
t2-t1

t1 = Sys.time()
ext_precip <- raster::extract(precipread_2, # Use the NL only temperature data
                            FMDt2, # Use the FMD that is reprojected into the right coordinate system for the raster
                            method = "simple") # takes a short time (~20 seconds)
t2 = Sys.time()
t2-t1

# Confirm the lengths match
c(length(ext_temp),length(ext_precip),length(unique(FMDt2$FMD)))

# [CAN BE SKIPPED IF FMD_abiotic FILE EXISTS] New data extraction for topography ----

shape4 %>% select(FMD, geometry) %>% write_sf("Data/FMD.shp")

shape4 %>% select(FMD, geometry) %>% st_union() %>% ggplot() + geom_sf()

elevation1 = raster("Data/Topodata/TOPO1/dataset/DEM.tif")
elevation2 = raster("Data/Topodata/TOPO2/dataset/DEM.tif")
elevation3 = raster("Data/Topodata/TOPO3/dataset/DEM.tif")
elevation4 = raster("Data/Topodata/TOPO4/dataset/DEM.tif")
elevation5 = raster("Data/Topodata/TOPO5/dataset/DEM.tif")

elevation = sp::merge(elevation1, elevation5)
elevation = sp::merge(elevation, elevation2)
elevation = sp::merge(elevation, elevation3)
elevation = sp::merge(elevation, elevation4)
plot(elevation)
writeRaster(elevation, "Data/elevation/elevation.grd")
rm(elevation1, elevation2, elevation3, elevation4, elevation5)

elevation = raster("Data/elevation/elevation.grd")

crs(elevation)
crs(shape4)
# Convert FMD to elevation coordinate reference system
FMD_elevationcoord = st_transform(shape4,crs(elevation)) 

elevation_2 = crop(elevation, extent(FMD_elevationcoord))

elevation_2 = df_spatial(elevation_2)

p_elevation = ggplot(FMD_elevationcoord) + theme_classic() + geom_raster(aes(x = x, y= y, fill = band1), data = elevation_2) + geom_sf(fill = NA, color = "white") + 
  scale_fill_viridis_c(name = "Elevation (m)") + theme(legend.position ="top")

tempread_2_1 = tempread_2/10
png("Plots/FMD_abiotic_c.png", width = 10, height = 10, units = "in", res = 600)
par(mfrow=c(2,2))
plot(tempread_2_1, main = "Temperature (\u00B0C)")
plot(precipread_2, main = "Precipitation (mm)")
plot(elevation_2, main = "Elevation (m)")
dev.off()

rm(tempread_2_1)

# Extract elevation data
ext_elevation <- raster::extract(elevation, # Use the NL elevation raster
                                 FMD_elevationcoord, # Use the FMD that is reprojected into the right coordinate system for the raster
                            method = "simple") # takes a longer time (hours)

write_rds(ext_elevation, "Data/ext_elevation.rds")

# [CAN BE SKIPPED IF FMD_abiotic FILE EXISTS] Create FMD_abiotic file
if(!exists("ext_elevation")){
  ext_elevation = read_rds("Data/ext_elevation.rds")
}
(meanelevation = do.call("rbind",lapply(ext_elevation, mean, na.rm = T)))
(meantemp = do.call("rbind", lapply(ext_temp, FUN = mean, na.rm = T)))
(meanprecip = do.call("rbind", lapply(ext_precip, FUN = mean, na.rm = T)))
(meantemp = meantemp/10) # Convert to Celsius (same storage conversion as before)

# Save a data frame so these calculations don't need to be repeated every time.

data.frame(Elevation = meanelevation,
           temp = meantemp,
           precip = meanprecip) %>%
  write_csv("Data/FMD_abiotic.csv")
