# Forestry Inventory at the spatial polygon level
# Author: Robert W. Buchkowski
# Date: Jan, 15/2021

# Goal: Extract precipitation and temperature data from the FID

# Load in libraries
library(raster)
library(sf)
library(tidyverse)

# Assign unique identifier and split the data into spatial and aspatial components. Only needs to happen once.
if(F){
  # Look at the layers in the forest inventory data set
  st_layers("C:/Users/rober/Documents/Moose_Model_Original_Data/Forest Inventory/Forestry.gdb")
  
  # Load the forest inventory data set
  forestry <- read_sf("C:/Users/rober/Documents/Moose_Model_Original_Data/Forest Inventory/Forestry.gdb", layer = "Updated_Forest")
  
  # Assign the forestry polygons a unique identifier and split into spatial and the data components
  
  forestry$UID = 1:dim(forestry)[1] # This UID will act as a reference throughout the analysis
  
  # Spatial forestry data set
  forestrysp <- forestry %>% select(UID, SHAPE)
  # split into three for saving the result:
  sect <- round(dim(forestrysp)[1]/3)
  
  write_sf(forestrysp[1:sect,], "Data/forest_inventory/inventory_spatial/forestry1.shp")
  write_sf(forestrysp[(sect + 1):(2*sect),], "Data/forest_inventory/inventory_spatial/forestry2.shp")
  write_sf(forestrysp[(2*sect+1):dim(forestrysp)[1],], "Data/forest_inventory/inventory_spatial/forestry3.shp")
  
  # Aspatial forestry data set
  forestryasp <- as.data.frame(forestry) %>% select(-SHAPE)
  write_rds(forestryasp, "Data/forest_inventory/forestry.rds")
  rm(forestryasp)
  
  extent(forestrysp)
  crs(forestrysp)
}

# Analysis will cut up different parts of the island and then extract elevation data by polygon.
forestry <- read_sf("Data/forest_inventory/inventory_spatial/forestry1.shp") %>%
  bind_rows(
    read_sf("Data/forest_inventory/inventory_spatial/forestry2.shp")
  ) %>%
  bind_rows(
    read_sf("Data/forest_inventory/inventory_spatial/forestry3.shp")
  )

# Build a series of forestry inventory subsections to analyze ----
npoly <- dim(forestry)[1]

length(seq(1,1849414, by = 18700))
max(seq(1,1849414, by = 18700))

seqpoly <- seq(1,npoly, by = 18700)
seqpoly <- c(seqpoly, npoly+1)

for(i in 1:(length(seqpoly)-1)){
  print(paste(seqpoly[i],(seqpoly[i+1]-1)))
  st_write(forestry[seqpoly[i]:(seqpoly[i+1]-1),], 
           paste0("Data/FRI_subset/FRIsubset_",i,".shp"))
}
rm(npoly, seqpoly)

# Extract the elevation data ----

# Read in the elevation data
elevation = raster("Data/Elevation/elevation.grd")

for(i in 1:99){
  
  forestry2 <- read_sf(paste0("Data/FRI_subset/FRIsubset_",i,".shp"))
  
  emptyq <- st_dimension(forestry2)
  
  forestry2 <- forestry2[!is.na(emptyq),]
  
  # forestry2 <- st_make_valid(forestry2)
  
  t1 = Sys.time()
  forestry2 <- raster::extract(elevation, # Use the NL only temperature data
                               forestry2, # Use the FMD that is reprojected into the right coordinate system for the raster
                               method = "simple", 
                               small = T, # Allow the code to find a possible value outside the polygon if it doesn't cover a raster cell
                               fun = mean, # Take the mean of multiple overlapping raster cells
                               sp = T #add the extracted values to the forestry2 data frame
  ) # takes a longer time 
  
  t2 = Sys.time()
  print(paste("Done",i, "in", t2-t1)) 
  
  write_rds(data.frame(forestry2), paste0("Data/FRI_elevation_extraction/FRI_subset_", i, ".rds"))
}

# Example for finding the nearest non-NA raster point
# https://stackoverflow.com/questions/27562076/if-raster-value-na-search-and-extract-the-nearest-non-na-pixel

#  Load in the results from the basic matching for forestry inventory and elevation: 
elevation_extract <- vector(mode = "list", length = 99)
for(i in 1:99) elevation_extract[[i]] = read_rds(paste0("Data/FRI_elevation_extraction/FRI_subset_", i, ".rds"))

elevation_extract <- do.call("rbind", elevation_extract)
colnames(elevation_extract)[2] = 'elevation'
write_rds(elevation_extract, paste0("Data/FRI_elevation_extraction/elevation_extract.rds")) # Write this data

# Identify the UIDs without data
forestry = forestry %>%
  left_join(elevation_extract, by = "UID")

forestry %>% filter(is.na(elevation)) %>% View()
# All the unfilled data are from EMPTY polygons, so we have elevation data for everything.

# Extract and load in temperature and precipitation data -----

# Extract temperature and precipitation data:

# These data come from: dan.mckenney@canada.ca
# I am using the bio parameters, which summarize the 30 year average. 

# Temperature (need to divide by 10)
tempread <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_01.tif")

# Summer Temperature (need to divide by 10)
tempreads <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_10.tif")

# Winter Temperature (need to divide by 10)
tempreadw <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_11.tif")

# Precipitation
precipread <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_12.tif")

# Precipitation
precipreads <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_18.tif")

# Precipitation
precipreadw <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_19.tif")

# Combine the temperature and precipitation rasters
tempprecip <- brick(tempread,tempreads,tempreadw, precipread, precipreads, precipreadw)
rm(tempread,tempreads,tempreadw, precipread, precipreads, precipreadw)

# Check that the CRS is the same for elevation
crs(elevation)
crs(tempprecip)

# Crop the raster to the right extent
temprecip <- crop(tempprecip,elevation)
rm(tempprecip)
gc()

# Extract by forestry polygon: 

for(i in 1:99){
  
  forestry2 <- read_sf(paste0("Data/FRI_subset/FRIsubset_",i,".shp"))
  
  emptyq <- st_dimension(forestry2)
  
  forestry2 <- forestry2[!is.na(emptyq),]
  
  t1 = Sys.time()
  forestry2 <- raster::extract(temprecip, # Use the NL only temperature and precipitation data
                               forestry2, # Use the FMD that is reprojected into the right coordinate system for the raster
                               method = "simple", 
                               small = T, # Allow the code to find a possible value outside the polygon if it doesn't cover a raster cell
                               fun = mean, # Take the mean of multiple overlapping raster cells
                               na.rm = T,
                               sp = T #add the extracted values to the forestry2 data frame
  ) # takes a longer time 
  
  t2 = Sys.time()
  print(paste("Done",i, "in", t2-t1)) 
  
  write_rds(data.frame(forestry2), paste0("Data/FRI_climate_extraction/FRI_subset_", i, ".rds"))
}

# Compile the extracted temperature and precipitation data
tp_extract <- vector(mode = "list", length = 99)
for(i in 1:99) tp_extract[[i]] = read_rds(paste0("Data/FRI_climate_extraction/FRI_subset_", i, ".rds"))

tp_extract <- do.call("rbind", tp_extract)
colnames(tp_extract)[2:7] = c('tempx10','tempSx10','tempWx10', 'precipmm','precipSmm','precipWmm')
write_rds(tp_extract, paste0("Data/FRI_climate_extraction/climate_extract.rds")) # Write this data

# The 2803 NAs for temperature and precipitation overlap
table(is.na(tp_extract$tempx10), is.na(tp_extract$precipmm),is.na(tp_extract$tempSx10), is.na(tp_extract$precipSmm),is.na(tp_extract$tempWx10), is.na(tp_extract$precipWmm))

# Identify the UIDs without data (2803)
forestry = forestry %>%
  left_join(
    tp_extract, by = "UID") %>% 
  filter(is.na(tempx10))

forestry = forestry[!is.na(st_dimension(forestry)),]

# Find the raster points in a buffer around each UID without a value for temperature and precipitation

forestry2 <- raster::extract(temprecip, # Use the NL only temperature and precipitation data
                             st_centroid(forestry[,1:2]), # Use the FMD that is reprojected into the right coordinate system for the raster
                             method = "simple", 
                             small = T, # Allow the code to find a possible value outside the polygon if it doesn't cover a raster cell
                             fun = mean, # Take the mean of multiple overlapping raster cells
                             na.rm = T,
                             buffer = 4400, # This buffer is chosen to have the minimum buffer that finds a non-zero raster value for each of the points in the data set
                             sp = T #add the extracted values to the forestry2 data frame
) # takes a longer time 

# Check to make sure the values are all there
any(is.na(forestry2$bio60_01))
any(is.na(forestry2$bio60_12))

tp_extract = tibble(tp_extract) %>%
  left_join(
    data.frame(forestry2) %>% select(UID, `bio60_01`, `bio60_10`,`bio60_11`,`bio60_12`,`bio60_18`,`bio60_19`)
  )

tp_extract = tp_extract %>% 
  mutate(flagTP = ifelse(is.na(tempx10), 2,1)) %>%
  mutate(tempx10 = ifelse(is.na(tempx10), `bio60_01`, tempx10)) %>%
  mutate(tempSx10 = ifelse(is.na(tempSx10), `bio60_10`, tempSx10)) %>%
  mutate(tempWx10 = ifelse(is.na(tempWx10), `bio60_11`, tempWx10)) %>%
  mutate(precipmm = ifelse(is.na(precipmm ), `bio60_12`, precipmm )) %>%
  mutate(precipSmm = ifelse(is.na(precipSmm ), `bio60_18`, precipmm )) %>%
  mutate(precipWmm = ifelse(is.na(precipWmm ), `bio60_19`, precipmm )) %>%
  mutate(temp = tempx10/10) %>% # Rescale temperature to the right units
  mutate(tempS = tempSx10/10) %>% # Rescale temperature to the right units
  mutate(tempW = tempWx10/10) %>% # Rescale temperature to the right units
  select(UID, temp,tempS,tempW, precipmm, precipSmm, precipWmm, flagTP)

tp_extract %>%
  pivot_longer(-UID) %>%
  filter(is.na(value))

write_rds(tp_extract, paste0("Data/FRI_climate_extraction/climate_extract.rds")) # Write this data