# Geological Analysis:

# Analysis of Newfoundland's Geological Data.
# Data set can be retrieved from: https://gis.geosurv.gov.nl.ca/
# Cannot be digitally distributed as per agreement, so I cannot provide it directly here.

# Goal is to determine the geological class for each forestry polygon

# Load in the libraries:
library(sf)
library(ggspatial)
library(raster)
library(tidyverse)

# Bedrock geology ----

# Load in the geological data

geology <- read_sf("C:/Users/rober/Documents/Moose_Model_Original_Data/Geology/Bedrock/Newfoundland_Detailed_Bedrock_Geology.shp")

# DESCRIPTION:
# The Detailed Bedrock Geology (polygon) layer is accessed through the Bedrock Geology group. This compilation, from Open File NFLD/2616 version 7, is presently available for all of Newfoundland (Nov. 2012). For the 'Detailed' level of geology, every unit that was distinguished on the original published maps is distinguished on the digital map. Fields on the polygon attribute table starting with D_ are specific to this map. For the 'General' level of geology, units are labelled according to the most senior stratigraphic division to which they belong. Fields on the polygon attribute table starting with G_ are specific to this map. For the 'Lithofacies' level of geology, units are slightly modified from those on the printed 1:1 million map (Colman-Sadd et al., 1990). The line work is the same as for the detailed version except that contacts within lumped units are omitted. Fields on the polygon attribute table starting with L_ are specific to this map. For the 'Tectonic' level of geology, the classification is similar to that used in the Western Cordillera (Gabrielse et al., 1991) and distinguishes terranes in more detail than the lithofacies version. Fields on the polygon attribute table starting with A_ are specific to this map. For more information, see this layer's Help file on the Geoscience Atlas (http://geoatlas.gov.nl.ca/) including a link to a file containing the ESRI layer symbology files.&lt;/SPAN&gt;&lt;/P&gt;&lt;/DIV&gt;&lt;/DIV&gt;</idAbs><idPurp>To provide a consistent detailed bedrock geology layer of Newfoundland and Labrador.</idPurp><idCredit>See the Help file for credit information, or contact: GeoScienceOnline@gov.nl.ca Geological Survey Division Department of Natural Resources Government of Newfoundland and Labrador St. John’s, NL, Canada

# Plot the geology
png("Plots/geology.png", width = 10, height = 10, units = "in", res = 600)
geology %>% ggplot() + geom_sf(aes(fill = L_TECTDIV), size = 0.1, color = "white") + theme_classic() + theme(legend.position = "none")
dev.off()

# Many geological features to choose from. The most general appears to be L_TECTDIV, but D_ROCKTYPE looks OK as well. Continuous variables include AGEBASE and AGETOP.

# Save a geology database that doesn't have spatial

geology %>% tibble() %>% select(-geometry) %>%
  write_rds("Data/geology_aspatial.rds")

# Link these geological features to the forestry plots:

# First add a unique key:
geology = geology %>% mutate(UIDg = 1:16725)

# Load in the forestry shapefile example to get CRS
aa = st_read("C:/Users/rober/Documents/GitHubRepos/MooseEcon/Data/forest_inventory/inventory_spatial/forestry1.shp", query = 'SELECT * FROM "forestry1" WHERE FID = 1')

# Need to transform to the same coordinate system
rm(aa)

# Get rid of the extra columns for now
geologyt = geology %>% select(UIDg)
rm(geology)

# Load in each slice of the forestry data and match it to a watershed
# .....(1)......
bb = st_read("C:/Users/rober/Documents/GitHubRepos/MooseEcon/Data/forest_inventory/inventory_spatial/forestry1.shp")

# Transform CRS:
bb <- st_transform(bb,crs(geologyt))

# Find the centroids of each polygon in the forestry data set
cc <- st_centroid(bb)

# Determine which point is inside which polygon
dd <- st_join(cc, geologyt)

# Check the NAs
dd2 = dd %>% filter(is.na(UIDg))

# Remove the NA dimensions
dd2 = dd2[!is.na(st_dimension(dd2)),]

# Find the ~2000 empty points:
bb2 <- bb %>% filter(UID %in% dd2$UID)

geologyt1 <- st_make_valid(geologyt)
geologyt2 <- st_crop(geologyt1, xmin = 452955, xmax = 647888, ymin = 5344153, ymax = 5716127)

# Find the matches for geology
dd3 <- st_join(bb2, geologyt2)

# If there are multiple, pick the one that is the biggest overlap
dd3 <- dd3 %>% mutate(AREA = st_area(dd3$geometry))

dd4 <- data.frame(dd3) %>% select(-geometry)

dd4 <- tibble(dd4) %>% 
  arrange(-AREA) %>%
  group_by(UID) %>% 
  slice(c(1))

unique(table(dd4$UID))
  
# Find the still remaining empty points:
dd5 <- dd4 %>% select(-AREA) %>% filter(is.na(UIDg))
bb2 <- bb %>% filter(UID %in% dd5$UID)

geologyt2 <- st_crop(geologyt1, st_bbox(bb2))
geologyt2_mod <- data.frame(geologyt2) %>% select(UIDg)

for(i in 1:dim(bb2)[1]){
  tmp1 <- st_distance(geologyt2,bb2[i,])
  
  dd5[i,"UIDg"] = geologyt2_mod[which.min(tmp1),"UIDg"]
}

# Merge these new IDs into the file:
dd = dd %>% 
  left_join(
    dd4 %>% select(-AREA) %>% rename(UIDg2 = UIDg)
  ) %>% 
  left_join(
    dd5 %>% rename(UIDg3 = UIDg)
  )

# Make a flag for how the data were calculated:
# 1 = point match, 2 = polygon overlap, 3 = nearest polygon

ddf = dd %>%
  mutate(flagUIDg = 
           ifelse(!is.na(UIDg), 1,
                  ifelse(!is.na(UIDg2), 2,3))
         ) %>%
  mutate(UIDg =
           ifelse(!is.na(UIDg), UIDg,
                  ifelse(!is.na(UIDg2), UIDg2,UIDg3))
  )

# Save the results
ee = tibble(ddf) %>% select(UID, UIDg,flagUIDg)

# Clean up
rm(bb,cc,dd, bb2, dd2, dd3, dd4, dd5, ddf,geologyt1, geologyt2, geologyt2_mod, tmp1)
gc()

# .....(2)......
bb = st_read("C:/Users/rober/Documents/GitHubRepos/MooseEcon/Data/forest_inventory/inventory_spatial/forestry2.shp")

# Transform CRS:
bb <- st_transform(bb,crs(geologyt))

# Find the centroids of each polygon in the forestry data set
cc <- st_centroid(bb)

# Determine which point is inside which polygon
dd <- st_join(cc, geologyt)

# Check the NAs
dd2 = dd %>% filter(is.na(UIDg))

# Remove the NA dimensions
dd2 = dd2[!is.na(st_dimension(dd2)),]

# Find the ~2000 empty points:
bb2 <- bb %>% filter(UID %in% dd2$UID)

geologyt1 <- st_make_valid(geologyt)
geologyt2 <- st_crop(geologyt1, st_bbox(bb2))

# Find the matches for geology
dd3 <- st_join(bb2, geologyt2)

# If there are multiple, pick the one that is the biggest overlap
dd3 <- dd3 %>% mutate(AREA = st_area(dd3$geometry))

dd4 <- data.frame(dd3) %>% select(-geometry)

dd4 <- tibble(dd4) %>% 
  arrange(-AREA) %>%
  group_by(UID) %>% 
  slice(c(1))

unique(table(dd4$UID))

# Find the still remaining empty points:
dd5 <- dd4 %>% select(-AREA) %>% filter(is.na(UIDg))
bb2 <- bb %>% filter(UID %in% dd5$UID)

geologyt2 <- st_crop(geologyt1, st_bbox(bb2))
geologyt2_mod <- data.frame(geologyt2) %>% select(UIDg)

for(i in 1:dim(bb2)[1]){
  tmp1 <- st_distance(geologyt2,bb2[i,])
  
  dd5[i,"UIDg"] = geologyt2_mod[which.min(tmp1),"UIDg"]
}

# Merge these new IDs into the file:
dd = dd %>% 
  left_join(
    dd4 %>% select(-AREA) %>% rename(UIDg2 = UIDg)
  ) %>% 
  left_join(
    dd5 %>% rename(UIDg3 = UIDg)
  )

# Make a flag for how the data were calculated:
# 1 = point match, 2 = polygon overlap, 3 = nearest polygon

ddf = dd %>%
  mutate(flagUIDg = 
           ifelse(!is.na(UIDg), 1,
                  ifelse(!is.na(UIDg2), 2,3))
  ) %>%
  mutate(UIDg =
           ifelse(!is.na(UIDg), UIDg,
                  ifelse(!is.na(UIDg2), UIDg2,UIDg3))
  )
# Save the results
ee = ee %>% 
  bind_rows(
    tibble(ddf) %>% select(UID, UIDg,flagUIDg)
  )

# Clean up
rm(bb,cc,dd, bb2, dd2, dd3, dd4, dd5, ddf, geologyt2, geologyt2_mod, tmp1)
gc()

# .....(3)......
bb = st_read("C:/Users/rober/Documents/GitHubRepos/MooseEcon/Data/forest_inventory/inventory_spatial/forestry3.shp")

# Transform CRS:
bb <- st_transform(bb,crs(geologyt))

# Find the centroids of each polygon in the forestry data set
cc <- st_centroid(bb)

# Determine which point is inside which polygon
dd <- st_join(cc, geologyt)

# Check the NAs
dd2 = dd %>% filter(is.na(UIDg))

# Remove the NA dimensions
dd2 = dd2[!is.na(st_dimension(dd2)),]

# Find the ~2000 empty points:
bb2 <- bb %>% filter(UID %in% dd2$UID)

geologyt1 <- st_make_valid(geologyt)
geologyt2 <- st_crop(geologyt1, st_bbox(bb2))

# Find the matches for geology
dd3 <- st_join(bb2, geologyt2)

# If there are multiple, pick the one that is the biggest overlap
dd3 <- dd3 %>% mutate(AREA = st_area(dd3$geometry))

dd4 <- data.frame(dd3) %>% select(-geometry)

dd4 <- tibble(dd4) %>% 
  arrange(-AREA) %>%
  group_by(UID) %>% 
  slice(c(1))

unique(table(dd4$UID))

# Find the still remaining empty points:
dd5 <- dd4 %>% select(-AREA) %>% filter(is.na(UIDg))
bb2 <- bb %>% filter(UID %in% dd5$UID)

geologyt2 <- st_crop(geologyt1, st_bbox(bb2))
geologyt2_mod <- data.frame(geologyt2) %>% select(UIDg)

for(i in 1:dim(bb2)[1]){
  tmp1 <- st_distance(geologyt2,bb2[i,])
  
  dd5[i,"UIDg"] = geologyt2_mod[which.min(tmp1),"UIDg"]
}

# Merge these new IDs into the file:
dd = dd %>% 
  left_join(
    dd4 %>% select(-AREA) %>% rename(UIDg2 = UIDg)
  ) %>% 
  left_join(
    dd5 %>% rename(UIDg3 = UIDg)
  )

# Make a flag for how the data were calculated:
# 1 = point match, 2 = polygon overlap, 3 = nearest polygon

ddf = dd %>%
  mutate(flagUIDg = 
           ifelse(!is.na(UIDg), 1,
                  ifelse(!is.na(UIDg2), 2,3))
  ) %>%
  mutate(UIDg =
           ifelse(!is.na(UIDg), UIDg,
                  ifelse(!is.na(UIDg2), UIDg2,UIDg3))
  )
# Save the results
ee = ee %>% 
  bind_rows(
    tibble(ddf) %>% select(UID, UIDg,flagUIDg)
  )

# Clean up
rm(bb,cc,dd, bb2, dd2, dd3, dd4, dd5, ddf, geologyt2, geologyt2_mod, tmp1)
gc()
# Inspect the result
dim(ee)

# Confirm that no UIDg values are NA, the ones that exist are probably missing geometries
ee %>% filter(is.na(UIDg))
ee %>% filter(is.na(UID))

# Save the resulting data
ee %>% write_rds("Data/geology_foreach_UID.rds")