# Watershed analysis:
# Author: Robert W. Buchkowski
# Created: Nov 4, 2020
# Modified: Feb 19, 2021

# Analysis Goals are to load in the watershed data and associated it with the Forestry polygons so that I can use these data in the final analysis.
# Output is the data frame matching UID and DATASETNAM

# Second analysis goal is to extract the temperature, precipitation, and site quality data for each watershed. The product is a data frame with minimum, mean, and maximum values for each of these values to test in the model.

# Load in the libraries:
library(sf)
library(ggspatial)
library(raster)
library(tidyverse)

# Match the watershed to the forestry data ----
tmp <- "C:/Users/rober/Documents/Moose_Model_Original_Data/Hydrology/"

LF <- list.files(tmp, recursive = T)

LF2 <- rep(NA, length(LF))
LF1 <- strsplit(LF, ".", fixed = T)

# New file paths
for(i in 1:length(LF)) LF2[i] = LF1[[i]][1]

# Unzip all elevation files
for(i in 1:length(LF)) unzip(paste0(tmp,"/",LF[i]), exdir = paste0(tmp,"/",LF2[i]))

# Load all the file paths
LF <- list.files(tmp, recursive = T)

# Get rid of all the zip folders
LF = LF[!grepl(".zip",LF, fixed = T)]

# Take a look at the different layers:
st_layers(paste0(tmp,"/", LF[[1]]))

# Keep only the working unit boundaries:
LF = LF[grepl("WORKUNIT_LIMIT_2.shp",LF)]

# Load in the NHN
NHN <- read_sf(paste0(tmp, "/", LF[1]))
  
for(i in 2:length(LF)){
  NHN <- NHN %>%
    bind_rows(
      read_sf(paste0(tmp, "/", LF[i]))
    )
}

# Select the right data sets
NHN <- NHN %>% select(DATASETNAM, geometry)

ggplot(NHN) + geom_sf()


NLFD = read_sf("Data/FMDmap/FMDmap.shp") %>%
  summarise(BalsmFr = sum(BalsmFr), do_union = T)

NHN = NHN %>%
  st_transform(st_crs(NLFD))

st_intersection(NLFD, NHN) %>%
  select(-BalsmFr) %>%
  rename(NHNname = DATASETNAM) %>%
  write_sf("Data/NHN/NHN.shp")

# Load in the polygon data for the forestry plots by watershed and confirm that the centroid is in the watershed

# Load in the forestry shapefile example to get CRS
aa = st_read("Data/forest_inventory/inventory_spatial/forestry1.shp", query = 'SELECT * FROM "forestry1" WHERE FID = 1')

# Transform to the same coordinate system
NHNt = st_transform(NHN,crs(aa))
rm(aa)

# Load in each slice of the forestry data and match it to a watershed
# .....(1)......
bb = read_sf("Data/forest_inventory/inventory_spatial/forestry1.shp")

# Find the centroids of each polygon in the forestry data set
cc <- st_centroid(bb)

# Determine which point is inside which polygon
dd <- st_join(cc, NHNt)

# Save the results
ee = tibble(dd) %>% 
  select(UID, DATASETNAM) %>%
  rename(NHN = DATASETNAM)

# Clean up
rm(bb,cc,dd)
gc()

# .....(2)......
bb = read_sf("Data/forest_inventory/inventory_spatial/forestry2.shp")

# Find the centroids of each polygon in the forestry data set
cc <- st_centroid(bb)

# Determine which point is inside which polygon
dd <- st_join(cc, NHNt)

# Save the results
ee = ee %>% 
  bind_rows(
    tibble(dd) %>% 
      select(UID, DATASETNAM) %>%
      rename(NHN = DATASETNAM)
  )

# Clean up
rm(bb,cc,dd)
gc()

# .....(3)......
bb = read_sf("Data/forest_inventory/inventory_spatial/forestry3.shp")

# Find the centroids of each polygon in the forestry data set
cc <- st_centroid(bb)

# Determine which point is inside which polygon
dd <- st_join(cc, NHNt)

# Save the results
ee = ee %>% 
  bind_rows(
    tibble(dd) %>% 
      select(UID, DATASETNAM) %>%
      rename(NHN = DATASETNAM)
  )

# Clean up
rm(bb,cc,dd)
gc()

# Inspect the result
dim(ee)

# Look at the NA values
nhnNA = ee %>% filter(is.na(NHN))
ee %>% filter(is.na(UID))

# Save the resulting data
ee %>% write_rds("Data/NHNwatershed_foreach_UID.rds")

# ... Check the NA values in the forestry data set ----

# Load the full data set
forestry <- read_sf("Data/forest_inventory/inventory_spatial/forestry1.shp") %>%
  bind_rows(
    read_sf("Data/forest_inventory/inventory_spatial/forestry2.shp")
  ) %>%
  bind_rows(
    read_sf("Data/forest_inventory/inventory_spatial/forestry3.shp")
  )

# Subset the forestry data set to only the values that don't have a watershed
forestry = forestry %>% 
  filter(UID %in% nhnNA$UID)
gc()

forestry <- forestry[!is.na(st_dimension(forestry)),]

# All the left over plots are empty, so no issues.

# Extract the watershed temperature, precipitation, and site quality data ----

# Read in the elevation data
elevation = raster("Data/Elevation/elevation.grd")

# Temperature (need to divide by 10)
tempread <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_01.tif")

# Precipitation
precipread <- raster("C:/Users/rober/Documents/Moose_Model_Original_Data/Climate/1981-2010/bio1981-2010_60arcsec/bio60_12.tif")

# Combine the temperature and precipitation rasters
tempprecip <- brick(tempread, precipread)
rm(tempread, precipread)

# Check that the CRS is the same for elevation
crs(elevation)
crs(tempprecip)

# Crop the raster to the right extent
temprecip <- crop(tempprecip,elevation)
rm(tempprecip, elevation)
gc()


# Calculate the means for temperature and precipitation to correct them appropriately (See statistical_models_biomass.R)

read_rds("Data/forest_inventory/forestry_compiled_biomass.rds") %>%
  filter(SITECODE %in% c("P","M", "G", "H")) %>%
  mutate(SITECODE = factor(SITECODE, levels = c("P","M", "G", "H")))%>%
  filter(!is.na(temp)&!is.na(precipmm)&!is.na(elevation) & !is.na(bedrock) & !is.na(surfacerock)) %>%
  filter(!is.na(HGT_CODE)) %>%
  mutate(HGT_CODE = as.numeric(HGT_CODE)) %>% # Get ride of all stands small enough to be browsed
  mutate(browsable = HGT_CODE == 1) %>%
  # Calculate the means
  summarize(temp = mean(temp), precip = mean(precipmm))

# Mean temperature is 3.84
# Mean precipitation is 1219

# Extract the temperature and precipitation data for each watershed to prepare a distribution

NHN_extract <- raster::extract(temprecip, # Use the NL only temperature and precipitation data
                             NHN, # Use the NHN that is re-projected into the right coordinate system for the raster
                             method = "simple", 
                             small = T # Allow the code to find a possible value outside the polygon if it doesn't cover a raster cell
)

# Extract the vectors for each variable and get temperature in the right units
temperature <- vector(mode  = "list", length = length(NHN_extract))
precipitation <- temperature

for(i in 1:length(temperature)){
  temperature[[i]] = NHN_extract[[i]][,1]/10 - 3.19 # Subtract the means
  precipitation[[i]] = NHN_extract[[i]][,2] - 1219 # Subtract the means
}



# Combine the metrics I am interested in and produce a new tibble
NHN_extract = tibble(NHN) %>%
  select(-geometry) %>%
  mutate(temp_mean = sapply(temperature, mean, na.rm = T),
         precip_mean = sapply(precipitation, mean, na.rm = T),
         temp_min = sapply(temperature, min, na.rm = T),
         precip_min = sapply(precipitation, min, na.rm = T),
         temp_max = sapply(temperature, max, na.rm = T),
         precip_max = sapply(precipitation, max, na.rm = T),)

# ... Site quality present in each watershed and the mode ----

# Load in the forestry data set
FRIaspatial <- tibble(read_rds("Data/forest_inventory/forestry.rds")) %>%
  # Add in the watershed codes to the forestry data set
  left_join(
    read_rds("Data/NHNwatershed_foreach_UID.rds") 
  )

# Identify the number of each site quality within each watershed
SITEQUAL = FRIaspatial %>% 
  select(UID, NHN, SITECODE) %>%
  mutate(P = ifelse(SITECODE == "P",1,0),
         G = ifelse(SITECODE == "G",1,0),
         M = ifelse(SITECODE == "M",1,0),
         H = ifelse(SITECODE == "H",1,0)) %>%
  select(-SITECODE) %>%
  pivot_longer(P:H) %>%
  group_by(NHN, name) %>%
  summarize(value = sum(value)) %>%
  pivot_wider() 
# All watersheds have P, G, and M sites, but only some have H sites.

SITEQUAL %>%
  # Calculate the mode
  pivot_longer(-NHN) %>%
  group_by(NHN) %>%
  slice_max(value) %>%
  select(-value) %>%
  rename(Qualmode = name) %>%
  # Rejoin to the full quality data
  inner_join(
    SITEQUAL
  ) %>%
  filter(!is.na(NHN)) %>%
  # Join in the temperature and precipitation data
  inner_join(
    NHN_extract %>% rename(NHN = DATASETNAM)
  ) %>%
  write_rds("Data/NHN_abiotic_variables.rds")
