# Forestry Inventory at the spatial polygon level
# Author: Robert W. Buchkowski
# Date: Jan, 19/2021

# Goal: Produce a final forest inventory data set that includes all the spatial covariates and that is aspatial (temperature, precipitation, elevation, geology, watershed)

# Note: conversion from presence-absence to biomass will occur in the next script, because this only works for a subset of the data

# Load in libraries
library(raster)
library(sf)
library(tidyverse)

# Combine the forestry data with extracted data on climate, elevation, watershed, and geology


forestry <- tibble(read_rds("Data/forest_inventory/forestry.rds")) %>%
  
  # Add in the watershed codes to the forestry data set
  left_join(
    read_rds("Data/NHNwatershed_foreach_UID.rds") 
  ) %>%
  
  # Match up the bedrock geology data
  left_join(
    read_rds("Data/geology_foreach_UID.rds") %>% # Read in the matches between bedrock geology and UID
      left_join(
        read_rds("Data/geology_aspatial.rds") %>% # Match that with the ID data for those data. Using L_TECTDIV, see notes in geology_analysis.R
          select(UIDg, L_TECTDIV)
      ) %>%
      select(-UIDg) %>% # Don't need this code
      rename(bedrock = `L_TECTDIV`, flagBedrock = flagUIDg)
  )%>%
  
  # Match up the surficial geology data
  left_join(
    read_rds("Data/surfacegeology_foreach_UID.rds") %>% # Read in the matches between surficial geology and UID
      left_join(
        read_rds("Data/surfacegeology_aspatial.rds") %>% # Match that with the ID data for those data. Using GENETIC1MA, see notes in geology_analysis.R
          select(UIDsg, GENETIC1MA)
      ) %>%
      select(-UIDsg) %>% # Don't need this code
      rename(surfacerock = `GENETIC1MA`, flagSurfacerock = flagUIDsg)
  ) %>%
  
  # Add in the elevation data
  left_join(
    tibble(read_rds("Data/FRI_elevation_extraction/elevation_extract.rds"))
  ) %>%
  
  # Add in the temperature and precipitation data
  left_join(
    tibble(read_rds("Data/FRI_climate_extraction/climate_extract.rds"))
  ) %>%
  
  # Add wind speed data
  left_join(
    tibble(read_rds("Data/FRI_windspeed_extraction/windspeed_extract.rds"))
  )

# Look at the properties of the final data set

dim(forestry) # Dimensions

# Number of each UID
repeats <- forestry %>%
  select(UID) %>%
  group_by(UID) %>%
  summarize(N = n()) %>%
  filter(N > 1)

dim(repeats) # None

# Check for NA values: ONLY THE 99 data points that have no polygon data
table(is.na(forestry$temp),is.na(forestry$precipmm),
      is.na(forestry$NHN), is.na(forestry$bedrock),
      is.na(forestry$surfacerock), is.na(forestry$windspeed))

# Write the final file after removing the 99 rows without polygon data
forestry %>%
  filter(!is.na(temp)) %>%
  write_rds("Data/forest_inventory/forestry_compiled.rds")
