# Script for converting P-A of the different tree species into a biomass estimate.
# Author: Robert W. Buchkowski
# Date: Dec. 2020

# Analysis Rationale:
# 1. Estimate aboveground biomass using height bins and the equation 7 provided by Jucker et al. 2017. Assume a constant crown diameter by age bin of the stand.
# 2. Multiply crown density bin by the composition estimate for each species to get the proportion of land area in the stand covered by that tree species. Then multiply by area to get the cover.
# 3. Calculate the number of trees that could fit in that land area using the assumed crown diameter (assuming a circle and no overlap). The SHAPE_A in the forestry dataset is in meters^2
# 4. Multiply (1) by (3) to get a biomass estimate for the polygon.
# 5. Compare these estimates to the forest management district data to see whether they line up.

# Load in the libraries:
if(!require("pacman")) install.packages("pacman")
p_load(tidyverse, sf)

# Prepare for comparison with FMD----

# Match each polygon with the corresponding FMD to compare the predicted tonnes/ha with the actual tonnes/ha across the island:

# This only needs to be run once to create the output file.
if(F){
  FMD <- read_sf("Data/FMDmap/FMDmap.shp")
  FMD %>% ggplot() + geom_sf(aes(fill = BalsmFr)) + theme_classic() + scale_fill_viridis_c()
  
  # Load in the forestry spatial database
  forestryshape <- read_sf("Data/forest_inventory/inventory_spatial/forestry1.shp") %>%
    bind_rows(
      read_sf("Data/forest_inventory/inventory_spatial/forestry2.shp")
    ) %>%
    bind_rows(
      read_sf("Data/forest_inventory/inventory_spatial/forestry3.shp")
    )
  
  # Find the centroids for each forestry polygon for mapping to FMD:
  forestryshape <- st_centroid(forestryshape)
  
  FMD2 = FMD %>% select(FMD)
  
  FMD2 = st_transform(FMD2, st_crs(forestryshape))
  
  inventoryFMD <- st_join(
    forestryshape,
    FMD2
  )
  
  tibble(inventoryFMD) %>%
    select(FMD, UID) %>%
    write_rds("Data/UIDinFMD.rds")
  
  # Check for repeats:
  read_rds("Data/UIDinFMD.rds") %>%
    group_by(UID) %>%
    summarize(N = n()) %>%
    filter(N > 1)
  
  forestryshape %>% filter(UID == 413108) %>%
    ggplot() + geom_sf(data = FMD2) + geom_sf(size = 3, col = "red") 
  read_rds("Data/UIDinFMD.rds") %>%
    filter(UID == 413108)
  
  # Load in the polygon
  a = st_read("C:/Users/rober/Documents/GitHubRepos/MooseEcon/Data/forest_inventory/inventory_spatial/forestry1.shp", query = 'SELECT * FROM "forestry1" WHERE UID = 413108')
  
  FMD2 %>%
    filter(FMD %in% c("10", "12")) %>%
    ggplot() + geom_sf() + geom_sf(data = a, size = 1.2, col = "red")
  
  b = st_join(a,
              FMD2 %>%
                filter(FMD %in% c("10", "12"))
              )
  
  st_area(b$`_ogr_geometry_`)
  
  # It is literally in the overlap between 10 and 12. Just randomly assign it to one sample:
  sample(c(10,12), 1) # Assigned to # 10
  
  # Remove that row from the data set
  read_rds("Data/UIDinFMD.rds")  %>%
    filter(!(UID == 413108 & FMD == 12)) %>%
    write_rds("Data/UIDinFMD.rds")
  
  # Identify the points that have no FMD
  nomatches = read_rds("Data/UIDinFMD.rds") %>%
    filter(is.na(FMD)) %>%
    select(UID)
  
  nomatches = 
    forestryshape %>% filter(UID %in% nomatches$UID)
  
  # Remove the 99 empty plots
  nomatches = nomatches[!is.na(st_dimension(nomatches)),]
    
  # Look at the distribution of these plots
  FMD2 %>%
    ggplot() + geom_sf() + geom_sf(data = nomatches, size = 1.2, col = "red")
  
  # I can easily match these up with nearest distance
  matches <- tibble(UID = nomatches$UID, FMD = NA)
  
  for(i in 1:dim(nomatches)[1]){
    dist = st_distance(FMD2, nomatches[i,])
    matches$FMD[i] = FMD2$FMD[which.min(dist)]
  }
  
  ff <- read_rds("Data/UIDinFMD.rds") %>%
    filter(!(UID %in% matches$UID)) %>%
    bind_rows(
      matches
    ) 
  
  # Now only the 99 are left
  table(is.na(ff$FMD))
  
  ff %>%
    write_rds("Data/UIDinFMD.rds")
  
  # Clean up the environment
  rm(list = ls()); gc()
  
}

# Break up the species composition and score it using the key from the schema:
forestrysp <- read_rds("Data/forest_inventory/forestry_compiled.rds") %>% # Even though I am cutting out other data, work with the lastest file that has everything
  select(UID, SPCOMP) %>%
  separate(SPCOMP, into = letters[1:3], sep = c(2,4)) %>%
  mutate(b = ifelse(b == "", NA,b)) %>%
  mutate(c = ifelse(c == "", NA,c)) %>%
  mutate(N = ifelse(!is.na(c), 3, ifelse(!is.na(b),2,1)))

forestrysp = forestrysp %>%
  pivot_longer(a:c) %>%
  filter(!is.na(value)) %>%
  left_join(
    tibble(name = c("a", "a", "a", "b", "b", "c"),
           N = c(1,2,3,2,3,3),
           perBA = c("75-100", "50-100", "25-50","25-49", "25-33", "25-33"))
  )

forestrysp = forestrysp %>%
  select(UID, value, perBA) %>%
  filter(!(value) %in% c("No", "ne", "",NA)) %>%
  distinct() %>%
  pivot_wider(names_from = value, values_from = perBA)

forestry2 = read_rds("Data/forest_inventory/forestry_compiled.rds") %>%
  left_join(
    forestrysp
  )

rm(forestrysp)

# Calculate the aboveground biomass of a typical balsam fir tree ----

# The max of the last bin is estimated based on tallest balsam fir
heightcode <- tibble(HGT_CODE = as.character(seq(1,8, by = 1)),
                     min = c(0, 3.6, 6.6, 9.6, 12.6, 15.6, 18.6, 21.6),
                     max = c(3.5, 6.5, 9.5, 12.5, 15.5, 18.5, 21.5, 27)) %>%
  mutate(med_HGT = min + (max-min)/2)

# The maximum of the last bin is estimated based on: https://tidcf.nrcan.gc.ca/en/trees/factsheet/80
agecode <- tibble(AGE_CODE = as.character(c(seq(1,7, by = 1),9)),
                     min = c(1,21,41,61,81,101,121,-1),
                     max = c(20,40,60,80,100,120,150,-1))%>%
  mutate(med_AGE = min + (max-min)/2)

crowncode <- tibble(CDEN_CODE = as.character(seq(0,3, by = 1)),
                    min = c(-1, 75, 51, 26),
                    max = c(-1, 100, 75, 50))%>%
  mutate(med_CDEN = min + (max-min)/2)

# Get the species cover and add in the median values for the different codes

# Make crown diameter a function of age
agecode$crowndiameter = c(1,2,2.5,2.5,2.5,2.5,2.5,2.5)

forestry3 <- forestry2 %>%
  filter(!is.na(AGE_CODE) & !is.na(HGT_CODE) & !is.na(CDEN_CODE)) %>%
  select(UID,SHAPE_Area, AGE_CODE, HGT_CODE,CDEN_CODE,bF) %>%
  filter(!is.na(bF)) %>%
  separate(bF, into = c("min", "max"), sep = "-") %>%
  mutate(min = as.numeric(min), max = as.numeric(max)) %>%
  mutate(med_bF = min + (max-min)/2) %>%
  # Join in the different median values
  left_join(
    heightcode %>% select(HGT_CODE, med_HGT)
  )%>%
  left_join(
    agecode %>% select(AGE_CODE, med_AGE,crowndiameter)
  )%>%
  left_join(
    crowncode %>% select(CDEN_CODE, med_CDEN)
  ) %>%
  select(UID, SHAPE_Area, med_bF, med_HGT, med_AGE, med_CDEN,crowndiameter) %>%
  # Get ride of unknown catagroy lines:
  filter(med_bF >= 0 & med_HGT >= 0 & med_AGE >= 0 & med_CDEN >= 0 ) %>%
  # Appy eqn 7 from Jucker et al. 2017. Result is in kg.
  mutate(AGBperTree = 
           (0.016 + 0.093)*exp(0.204^2/2)*(med_HGT*crowndiameter)^(2.013-0.223)) %>%
  # Calculate the proportion of each stand area covered by balsam fir and then multiply by the stand area
  mutate(propcover = (med_CDEN/100)*(med_bF/100)) %>%
  
  # Calculate the number of trees/m2 for full coverage:
  mutate(trees_m2 = 1/3.14159*(crowndiameter/2)^2) %>%
  
  # Calculate the tree density taking into account cover reduction (trees/m2)
  mutate(treedensity = trees_m2*propcover) %>%
  
  # Calculate the estimated biomass
  mutate(AGBest = AGBperTree*treedensity) %>% # (kg/tree * tree/m2)
  # Convert kg/m2 to tonnes/ha by multiplying by 10,000 and dividing by 1000
  mutate(AGBest = AGBest*10000/1000)

# Sprugel 1984 suggests that between 50 and 100 kg is reasonable per Balsam fir tree when fully grown, so these estimates look OK.
forestry3 %>% 
  ggplot(aes(x = AGBperTree)) + 
  geom_histogram() + 
  theme_classic()

forestry3 %>% 
  group_by(med_AGE) %>%
  summarize(treedensity = mean(treedensity)) %>% 
  ggplot(aes(x = med_AGE, y = treedensity)) + 
  geom_point()+
  theme_classic() +
  scale_x_log10()

forestry3 %>% 
  group_by(med_AGE) %>%
  summarize(AGBest = mean(AGBest)) %>%
  ggplot(aes(x = med_AGE, y = AGBest)) + 
  geom_point() + 
  theme_classic()


# Save for a final file
balsamfir = forestry3 %>% select(UID, AGBest) %>% rename(bF = AGBest)

# ... Compare balsam fir to FMD measures ----

# First associated each measurement with an FMD and take the average
sumFMD = forestry3 %>%
  left_join(
    read_rds("Data/UIDinFMD.rds")
  ) %>%
  group_by(FMD) %>%
  summarize(AGB = weighted.mean(AGBest, SHAPE_Area), 
            sd = sd(AGBest), 
            med = median(AGBest))

# Join the predictions with the actual dataset by FMD:
FMD <- read_sf("Data/FMDmap/FMDmap.shp") %>% 
  left_join(
    sumFMD
  )

m1 <- lm(AGB~BalsmFr + 0, data = FMD); summary(m1)

p1 = agecode %>%
  filter(med_AGE > 0) %>%
  ggplot(aes(x = med_AGE, y = crowndiameter)) + 
  geom_line(size = 1.5) +
  theme_classic() +
  xlab("Median Age (years)") +
  ylab("Crown diameter (m)") +
  ggtitle("Age to Crown diameter Conversion")

p2 = tibble(expand.grid(med_HGT= heightcode$med_HGT, crowndiameter = agecode$crowndiameter)) %>%
  mutate(AGBperTree = 
           (0.016 + 0.093)*exp(0.204^2/2)*(med_HGT*crowndiameter)^(2.013-0.223)) %>%
  ggplot(aes(x = med_HGT, y = AGBperTree, color = crowndiameter, group = crowndiameter)) + 
  geom_line(size = 1.5) +
  theme_classic() +
  scale_color_viridis_c(name = "Crown\ndiameter\n(m)") + 
  xlab("Median Height (m)") +
  ylab("Est. abovground biomass (kg)") +
  ggtitle("Jucker et al. 2017 Equation")

p3 = FMD %>% 
  ggplot(aes(x = BalsmFr, y = AGB)) + 
  theme_classic() +
  stat_smooth(method = "lm", formula = y~x + 0 ) +
  geom_pointrange(aes(ymin = AGB, ymax = AGB+sd), size = 1.5) +
  xlab("Measured Balsam Fir Biomass") +
  ylab("Predicted Balsam Fir Biomass")  +
  geom_abline(intercept = 0, slope = 1, linetype = 2) +
  geom_text(aes(label = t), parse = T,
            data = tibble(BalsmFr = c(30,30,30,30),
                          AGB = c(130,120,110,100),
                          t = c("beta==1.02%+-%0.05",
                                "beta[0]%==%0",
                                "R^2==0.95",
                                "italic(p)<1%*%10^-4")))

# Calculate the aboveground biomass of a typical black spruce tree ----

# The max of the last bin is estimated based on tallest back spruce
heightcode <- tibble(HGT_CODE = as.character(seq(1,8, by = 1)),
                     min = c(0, 3.6, 6.6, 9.6, 12.6, 15.6, 18.6, 21.6),
                     max = c(3.5, 6.5, 9.5, 12.5, 15.5, 18.5, 21.5, 30)) %>%
  mutate(med_HGT = min + (max-min)/2)

# The maximum of the last bin is estimated based on: https://tidcf.nrcan.gc.ca/en/trees/factsheet/39
agecode <- tibble(AGE_CODE = as.character(c(seq(1,7, by = 1),9)),
                  min = c(1,21,41,61,81,101,121,-1),
                  max = c(20,40,60,80,100,120,200,-1))%>%
  mutate(med_AGE = min + (max-min)/2)

crowncode <- tibble(CDEN_CODE = as.character(seq(0,3, by = 1)),
                    min = c(-1, 75, 51, 26),
                    max = c(-1, 100, 75, 50))%>%
  mutate(med_CDEN = min + (max-min)/2)

# Get the species cover and add in the median values for the different codes

# Make crown diameter a function of age
agecode$crowndiameter = c(1,2,2.5,2.5,2.5,2.5,2.5,2.5)

forestry3 <- forestry2 %>%
  filter(!is.na(AGE_CODE) & !is.na(HGT_CODE) & !is.na(CDEN_CODE)) %>%
  select(UID,SHAPE_Area, AGE_CODE, HGT_CODE,CDEN_CODE,bS) %>%
  filter(!is.na(bS)) %>%
  separate(bS, into = c("min", "max"), sep = "-") %>%
  mutate(min = as.numeric(min), max = as.numeric(max)) %>%
  mutate(med_bS = min + (max-min)/2) %>%
  # Join in the different median values
  left_join(
    heightcode %>% select(HGT_CODE, med_HGT)
  )%>%
  left_join(
    agecode %>% select(AGE_CODE, med_AGE,crowndiameter)
  )%>%
  left_join(
    crowncode %>% select(CDEN_CODE, med_CDEN)
  ) %>%
  select(UID, SHAPE_Area, med_bS, med_HGT, med_AGE, med_CDEN,crowndiameter) %>%
  # Get ride of unknown catagroy lines:
  filter(med_bS >= 0 & med_HGT >= 0 & med_AGE >= 0 & med_CDEN >= 0 ) %>%
  # Appy eqn 7 from Jucker et al. 2017. Result is in kg.
  mutate(AGBperTree = 
           (0.016 + 0.093)*exp(0.204^2/2)*(med_HGT*crowndiameter)^(2.013-0.223)) %>%
  # Calculate the proportion of each stand area covered by balsam fir and then multiply by the stand area
  mutate(propcover = (med_CDEN/100)*(med_bS/100)) %>%
  
  # Calculate the number of trees/m2 for full coverage:
  mutate(trees_m2 = 1/3.14159*(crowndiameter/2)^2) %>%
  
  # Calculate the tree density taking into account cover reduction (trees/m2)
  mutate(treedensity = trees_m2*propcover) %>%
  
  # Calculate the estimated biomass
  mutate(AGBest = AGBperTree*treedensity) %>% # (kg/tree * tree/m2)
  # Convert kg/m2 to tonnes/ha by multiplying by 10,000 and dividing by 1016.04691
  mutate(AGBest = AGBest*10000/1000)

forestry3 %>% 
  ggplot(aes(x = AGBest)) + 
  geom_histogram() + 
  theme_classic()

# Save for a final file
blackspruce  = forestry3 %>% select(UID, AGBest) %>% rename(bS = AGBest)

# ... Compare black spruce to FMD measures ----

# First associated each measurement with an FMD and take the average
sumFMD = forestry3 %>%
  left_join(
    read_rds("Data/UIDinFMD.rds")
  ) %>%
  group_by(FMD) %>%
  summarize(AGB = weighted.mean(AGBest, SHAPE_Area), 
            sd = sd(AGBest), 
            med = median(AGBest))

# Join the predictions with the actual dataset by FMD:
FMD <- read_sf("Data/FMDmap/FMDmap.shp") %>% 
  left_join(
    sumFMD
  )

m1 <- lm(AGB~BlckSpr + 0, data = FMD); summary(m1)

p4 = FMD %>% 
  ggplot(aes(x = BlckSpr, y = AGB)) + 
  theme_classic() +
  stat_smooth(method = "lm", formula = y~x + 0 ) +
  geom_pointrange(aes(ymin = AGB, ymax = AGB+sd), size = 1.5) +
  xlab("Measured Black Spruce Biomass") +
  ylab("Predicted Black Spruce Biomass") +
  geom_abline(intercept = 0, slope = 1, linetype = 2) +
  geom_text(aes(label = t), parse = T,
            data = tibble(BlckSpr = c(30,30,30,30),
                          AGB = c(130,120,110,100),
                          t = c("beta==1.10%+-%0.07",
                                "beta[0]%==%0",
                                "R^2==0.93",
                                "italic(p)<1%*%10^-4")))


# Calculate the aboveground biomass of a typical white spruce tree ----

# The max of the last bin is estimated based on tallest white spruce
heightcode <- tibble(HGT_CODE = as.character(seq(1,8, by = 1)),
                     min = c(0, 3.6, 6.6, 9.6, 12.6, 15.6, 18.6, 21.6),
                     max = c(3.5, 6.5, 9.5, 12.5, 15.5, 18.5, 21.5, 25)) %>%
  mutate(med_HGT = min + (max-min)/2)

# The maximum of the last bin is estimated based on: https://tidcf.nrcan.gc.ca/en/trees/factsheet/38
agecode <- tibble(AGE_CODE = as.character(c(seq(1,7, by = 1),9)),
                  min = c(1,21,41,61,81,101,121,-1),
                  max = c(20,40,60,80,100,120,200,-1))%>%
  mutate(med_AGE = min + (max-min)/2)

crowncode <- tibble(CDEN_CODE = as.character(seq(0,3, by = 1)),
                    min = c(-1, 75, 51, 26),
                    max = c(-1, 100, 75, 50))%>%
  mutate(med_CDEN = min + (max-min)/2)

# Get the species cover and add in the median values for the different codes

# Make crown diameter a function of age
agecode$crowndiameter = c(1,2,2.5,2.5,2.5,2.5,2.5,2.5)

forestry3 <- forestry2 %>%
  filter(!is.na(AGE_CODE) & !is.na(HGT_CODE) & !is.na(CDEN_CODE)) %>%
  select(UID,SHAPE_Area, AGE_CODE, HGT_CODE,CDEN_CODE,wS) %>%
  filter(!is.na(wS)) %>%
  separate(wS, into = c("min", "max"), sep = "-") %>%
  mutate(min = as.numeric(min), max = as.numeric(max)) %>%
  mutate(med_wS = min + (max-min)/2) %>%
  # Join in the different median values
  left_join(
    heightcode %>% select(HGT_CODE, med_HGT)
  )%>%
  left_join(
    agecode %>% select(AGE_CODE, med_AGE,crowndiameter)
  )%>%
  left_join(
    crowncode %>% select(CDEN_CODE, med_CDEN)
  ) %>%
  select(UID, SHAPE_Area, med_wS, med_HGT, med_AGE, med_CDEN,crowndiameter) %>%
  # Get ride of unknown catagroy lines:
  filter(med_wS >= 0 & med_HGT >= 0 & med_AGE >= 0 & med_CDEN >= 0 ) %>%
  # Appy eqn 7 from Jucker et al. 2017. Result is in kg.
  mutate(AGBperTree = 
           (0.016 + 0.093)*exp(0.204^2/2)*(med_HGT*crowndiameter)^(2.013-0.223)) %>%
  # Calculate the proportion of each stand area covered by balsam fir and then multiply by the stand area
  mutate(propcover = (med_CDEN/100)*(med_wS/100)) %>%
  
  # Calculate the number of trees/m2 for full coverage:
  mutate(trees_m2 = 1/3.14159*(crowndiameter/2)^2) %>%
  
  # Calculate the tree density taking into account cover reduction (trees/m2)
  mutate(treedensity = trees_m2*propcover) %>%
  
  # Calculate the estimated biomass
  mutate(AGBest = AGBperTree*treedensity) %>% # (kg/tree * tree/m2)
  # Convert kg/m2 to tonnes/ha by multiplying by 10,000 and dividing by 1016.04691
  mutate(AGBest = AGBest*10000/1000)

forestry3 %>% 
  ggplot(aes(x = AGBest)) + 
  geom_histogram() + 
  theme_classic()

# Save for a final file
whitespruce  = forestry3 %>% select(UID, AGBest) %>% rename(wS = AGBest)

# ... Compare white spruce to FMD measures ----

# First associated each measurement with an FMD and take the average
sumFMD = forestry3 %>%
  left_join(
    read_rds("Data/UIDinFMD.rds")
  ) %>%
  group_by(FMD) %>%
  summarize(AGB = weighted.mean(AGBest, SHAPE_Area), 
            sd = sd(AGBest), 
            med = median(AGBest))

# Join the predictions with the actual dataset by FMD:
FMD <- read_sf("Data/FMDmap/FMDmap.shp") %>% 
  left_join(
    sumFMD
  )

m1 <- lm(AGB~WhtSprc + 0, data = FMD); summary(m1)

p5 = FMD %>% 
  ggplot(aes(x = WhtSprc, y = AGB)) + 
  theme_classic() +
  stat_smooth(method = "lm", formula = y~x + 0 ) +
  geom_pointrange(aes(ymin = AGB, ymax = AGB+sd), size = 1.5) +
  xlab("Measured White Spruce Biomass") +
  ylab("Predicted White Spruce Biomass") +
  geom_abline(intercept = 0, slope = 1, linetype = 2) +
  geom_text(aes(label = t), parse = T,
            data = tibble(WhtSprc = c(4,4,4,4),
                          AGB = c(130,120,110,100),
                          t = c("beta==3.94%+-%0.85",
                                "beta[0]%==%0",
                                "R^2==0.54",
                                "italic(p)<1%*%10^-3")))


# Compare total spruce to FMD measures ----

# First associated each measurement with an FMD and take the average
sumFMD = whitespruce %>%
  full_join(
    blackspruce, by = "UID"
  ) %>%
  left_join(
    forestry2 %>% select(UID, SHAPE_Area), by = "UID"
  ) %>%
  mutate(wS = replace_na(wS, 0), bS = replace_na(bS,0)) %>%
  mutate(AGBest = wS + bS) %>% select(UID, AGBest, SHAPE_Area) %>%
  left_join(
    read_rds("Data/UIDinFMD.rds")
  ) %>%
  filter(!is.na(AGBest)) %>%
  group_by(FMD) %>%
  summarize(AGB = weighted.mean(AGBest, SHAPE_Area), 
            sd = sd(AGBest), 
            med = median(AGBest))

# Join the predictions with the actual dataset by FMD:
FMD <- read_sf("Data/FMDmap/FMDmap.shp") %>% 
  left_join(
    sumFMD
  ) %>%
  mutate(TotSprc = WhtSprc + BlckSpr)

m1 <- lm(AGB~TotSprc + 0, data = FMD); summary(m1)

p6 = FMD %>% 
  ggplot(aes(x = TotSprc, y = AGB)) + 
  theme_classic() +
  stat_smooth(method = "lm", formula = y~x + 0 ) +
  geom_pointrange(aes(ymin = AGB, ymax = AGB+sd), size = 1.5) +
  xlab("Measured Spruce Biomass") +
  ylab("Predicted Spruce Biomass") +
  geom_abline(intercept = 0, slope = 1, linetype = 2) +
  geom_text(aes(label = t), parse = T,
            data = tibble(TotSprc = rep(30,4),
                          AGB = c(130,120,110,100)-10,
                          t = c("beta==1.00%+-%0.054",
                                "beta[0]%==%0",
                                "R^2==0.95",
                                "italic(p)<1%*%10^-4")))

# Plot the resulting analysis -----
png("Plots/convert_to_biomass.png", width = 8, height = 12, units= "in", res = 600)
ggpubr::ggarrange(ggpubr::ggarrange(p1,p2), p3,p4,p5, nrow = 4, ncol = 1)
dev.off()

# With spruce together
png("Plots/convert_to_biomass_bF_S.png", width = 7, height = 8, units= "in", res = 600)
ggpubr::ggarrange(ggpubr::ggarrange(p1,p2), p3,p6, nrow = 3, ncol = 1)
dev.off()

# Save the biomass estimates as as new file ----

fullAGB = balsamfir %>% 
  full_join(blackspruce) %>% 
  full_join(whitespruce) %>%
  mutate(bF = replace_na(bF, 0)) %>%
  mutate(bS = replace_na(bS, 0)) %>%
  mutate(wS = replace_na(wS, 0))

fullAGB %>% write_rds("Data/biomass_estimates_full.rds")

# Save a final file that contains all the data as a rds file for use and a csv file for sharing.
finaldata <- read_rds("Data/forest_inventory/forestry_compiled.rds") %>%
  left_join(
    fullAGB %>%
      rename(bFbiomass = bF,
             bSbiomass = bS,
             wSbiomass = wS)
  )

finaldata %>% write_rds("Data/forest_inventory/forestry_compiled_biomass.rds")

finaldata %>% write_csv("Data/forest_inventory/forestry_compiled_biomass.csv")
